import os
import simplejson as json

from django.core.management.base import BaseCommand, CommandError

from tendenci.core.site_settings.models import Setting


class Command(BaseCommand):
    """
    Use this command to add a settings to the database

    Reads from the json file located in the same
    directory as this file and loops though it. If a setting
    exists it will be skipped. It checks if a setting exists
    by name, scope, and scope_category.

    Optional Command Arguments:
        `json`: path to the json file

    Json format:
        [
          {
            "name": "",
            "label": "",
            "description": "",
            "data_type": "",
            "value": "",
            "default_value": "",
            "input_type": "",
            "input_value": "",
            "client_editable": "",
            "store": "",
            "scope": "",
            "scope_category": "",
          }
        ]

    Json example:
        [
          {
            "name": "enabled",
            "label": "enabled",
            "description": "Module is enabled or not.",
            "data_type": "boolean",
            "value": "true",
            "default_value": "true",
            "input_type": "select",
            "input_value": "true, false",
            "client_editable": "1",
            "store": "1",
            "scope": "module",
            "scope_category": "memberships",
          }
        ]

    Json field definitions:
        `name`: The machine name. No spaces or special characters.
              Remember that this is what the python code uses to
              find the setting

        `label`: The human readable version of 'name'

        `description`: A non-html or html description of the setting.
                       Refer to the 'site_settings_setting' table for examples

        `data_type`: boolean or string

        `value`: the current value

        `default_value`: the original value

        `input_type`: select or text (used by the autogenerated interface)

        `input_value`: comma delimited list or just a string

        `store`: boolean value. Tell the system whether or not to
               cache the setting

        `scope`: site or module

        `scope_category`: this is the module name the settings belongs too.
                        refer to django contenttypes app_label. If this
                        is a settings that doesn't apply to a django
                        application use 'global'
    """
    help = 'Add a setting to the site_settings_setting table'

    def add_settings(self, settings):
        """
        Loop through the settings and add them
        """
        for setting in settings:
            new_setting = Setting(**setting)

            exists = Setting.objects.filter(**{
                'name': new_setting.name,
                'scope': new_setting.scope,
                'scope_category': new_setting.scope_category
            }).exists()

            if (exists):
                print '%s (%s) already exists ... skipping.' % (
                    new_setting.name,
                    new_setting.scope_category
                )
            else:
                print '%s (%s) ... done.' % (
                    new_setting.name,
                    new_setting.scope_category
                )
                new_setting.save()

    def handle(self, *args, **options):
        json_file = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            'add_settings.json')
        )

        if 'json' in options:
            json_file = options['json']

        if os.path.isfile(json_file):
            with open(json_file, 'r') as f:
                try:
                    settings = json.loads(f.read())
                except ValueError as e:
                    raise CommandError(e)
                self.add_settings(settings)
        else:
            raise CommandError('%s: Could not find json file %s' % (
                __file__,
                json_file,
            ))



from datetime import datetime, timedelta
from os.path import join, isdir
from os import mkdir
from PIL import Image

from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.http import HttpResponseRedirect, HttpResponse
from django.core.urlresolvers import reverse
from django.conf import settings
from django.db.models import Count

from tendenci.core.base.http import render_to_403
from tendenci.core.base.http import Http403
from tendenci.core.perms.utils import has_perm
from tendenci.core.perms.decorators import superuser_required
from tendenci.core.registry import site

from tendenci.core.event_logs.utils import day_bars, month_days,\
    request_month_range
from tendenci.core.event_logs.models import EventLog, EventLogBaseColor, EventLogColor
from tendenci.core.event_logs.forms import EventLogSearchForm, EventsFilterForm
from tendenci.core.event_logs.colors import non_model_event_logs, get_color


def index(request, id=None, template_name="event_logs/view.html"):
    if not id:
        return HttpResponseRedirect(reverse('event_log.search'))
    event_log = get_object_or_404(EventLog, pk=id)

    if has_perm(request.user, 'event_logs.view_eventlog'):
        return render_to_response(template_name, {'event_log': event_log},
            context_instance=RequestContext(request))
    else:
        raise Http403


def search(request, template_name="event_logs/search.html"):
    if not has_perm(request.user, 'event_logs.view_eventlog'):
        raise Http403

    event_logs = []

    # defaults for date information
    start_dt = datetime.now() - timedelta(weeks=4)
    end_dt = datetime.now()
    date_range_query = '&start_dt_0=%s&start_dt_1=%s&end_dt_0=%s&end_dt_1=%s'

    if request.GET:
        search_form = EventLogSearchForm(request.GET)
        if search_form.is_valid():
            # set default date range if they haven't specified it
            if not all([search_form.cleaned_data['start_dt'], search_form.cleaned_data['end_dt']]):
                search_form.cleaned_data['start_dt'] = start_dt
                search_form.cleaned_data['end_dt'] = end_dt

            # if they set the dates lets update the range
            start_dt = search_form.cleaned_data['start_dt']
            if isinstance(start_dt, unicode):
                start_dt = datetime.strptime(
                    start_dt,
                    '%Y-%m-%d %H:%M'
                )

            end_dt = search_form.cleaned_data['end_dt']
            if isinstance(end_dt, unicode):
                end_dt = datetime.strptime(
                    end_dt,
                    '%Y-%m-%d %H:%M'
                )

            event_logs = EventLog.objects.search(query=search_form)
    else:
        search_form = EventLogSearchForm()
        filters = {
            'create_dt__gte': start_dt,
            'create_dt__lte': end_dt
        }
        event_logs = EventLog.objects.filter(**filters)
        event_logs = event_logs.order_by('-create_dt')

    return render_to_response(template_name, {
        'event_logs': event_logs,
        'search_form': search_form,
        'date_range_query': date_range_query % (
            start_dt.strftime('%Y-%m-%d'),
            start_dt.strftime('%I:%M %p'),
            end_dt.strftime('%Y-%m-%d'),
            end_dt.strftime('%I:%M %p')
        )
        },
        context_instance=RequestContext(request))


def print_view(request, id, template_name="event_logs/print-view.html"):
    event_log = get_object_or_404(EventLog, pk=id)

    if has_perm(request.user, 'event_logs.view_eventlog'):
        return render_to_response(template_name, {'event_log': event_log},
            context_instance=RequestContext(request))
    else:
        raise Http403


@login_required
def colored_image(request, color):
    from webcolors import hex_to_rgb

    base_dir = join(settings.MEDIA_ROOT, 'event_logs')
    full_path = join(base_dir, '%s.png' % color)

    if hasattr(settings, 'USE_S3_STORAGE') and settings.USE_S3_STORAGE:
        rgb = hex_to_rgb('#%s' % color)
        image = Image.new('RGB', (1, 1), rgb)
        response = HttpResponse(mimetype="image/png")
        image.save(response, "PNG")
        return response
    else:
        # make the dir if it doesn't exist
        if not isdir(base_dir):
            mkdir(base_dir)

        try:
            f = open(full_path, 'rb')
            data = f.read()
            f.close()
        except:
            rgb = hex_to_rgb('#%s' % color)
            image = Image.new('RGB', (1, 1), rgb)
            image.save(full_path, "PNG")
            f = open(full_path, 'rb')
            data = f.read()
            f.close()

    return HttpResponse(data, mimetype="image/png")


def source_colors(data):
    for item in data:
        item['color'] = EventLogBaseColor.get_color(item['source'])


def application_colors(data):
    for item in data:
        item['color'] = EventLogBaseColor.get_color(item['application'])


def event_colors(data):
    for item in data:
        item['color'] = get_color(str(item['event_id']))


def action_colors(data):
    for item in data:
        item['color'] = get_color(str(item['action']))


@superuser_required
def event_summary_report(request):
    queryset = EventLog.objects.all()
    form = EventsFilterForm(request.GET)
    if form.is_valid():
        queryset = form.process_filter(queryset)

    from_date, to_date = request_month_range(request)
    next_day = to_date+timedelta(days=1)
    queryset = queryset.filter(create_dt__gte=from_date, create_dt__lte=next_day)

    chart_data = queryset\
                .extra(select={'day': 'DATE(create_dt)'})\
                .values('day', 'application')\
                .annotate(count=Count('pk'))\
                .order_by('day', '-count')
    chart_data = day_bars(chart_data, from_date.year, from_date.month, 300, application_colors)

    summary_data = queryset\
                .values('application')\
                .annotate(count=Count('pk'))\
                .order_by('-count')
    application_colors(summary_data)

    m = 1+len(summary_data)/3
    mm = 2*m
    summary_data = summary_data[:m], summary_data[m:mm], summary_data[mm:]

    return render_to_response(
                'reports/event_summary.html',
                {'chart_data': chart_data, 'summary_data': summary_data,
                 'form': form, 'date_range': (from_date, to_date)},
                context_instance=RequestContext(request))


@superuser_required
def event_application_summary_report(request, application):
    queryset = EventLog.objects.filter(application=application)
    form = EventsFilterForm(request.GET)
    if form.is_valid():
        queryset = form.process_filter(queryset)

    from_date, to_date = request_month_range(request)
    next_day = to_date+timedelta(days=1)
    queryset = queryset.filter(create_dt__gte=from_date, create_dt__lte=next_day)

    chart_data = queryset\
                .extra(select={'day': 'DATE(create_dt)'})\
                .values('day', 'action')\
                .annotate(count=Count('pk'))\
                .order_by('day', '-count')
    chart_data = day_bars(chart_data, from_date.year, from_date.month, 300, action_colors)

    summary_data = queryset\
                .values('action', 'description')\
                .annotate(count=Count('pk'))\
                .order_by('-count')
    action_colors(summary_data)

    return render_to_response(
                'reports/event_application_summary.html',
                {'chart_data': chart_data, 'summary_data': summary_data,
                 'form': form, 'date_range': (from_date, to_date),
                 'application': application},
                context_instance=RequestContext(request))


@superuser_required
def event_summary_historical_report(request):
    """
    This report queries based on source for historical reporting purposes
    """
    queryset = EventLog.objects.all()
    form = EventsFilterForm(request.GET)
    if form.is_valid():
        queryset = form.process_filter(queryset)

    from_date, to_date = request_month_range(request)
    next_day = to_date+timedelta(days=1)
    queryset = queryset.filter(create_dt__gte=from_date, create_dt__lte=next_day)

    chart_data = queryset\
                .extra(select={'day': 'DATE(create_dt)'})\
                .values('day', 'source')\
                .annotate(count=Count('pk'))\
                .order_by('day', '-count')
    chart_data = day_bars(chart_data, from_date.year, from_date.month, 300, source_colors)

    summary_data = queryset\
                .values('source')\
                .annotate(count=Count('pk'))\
                .order_by('-count')
    source_colors(summary_data)

    m = 1+len(summary_data)/3
    mm = 2*m
    summary_data = summary_data[:m], summary_data[m:mm], summary_data[mm:]

    return render_to_response(
                'reports/event_summary_historical.html',
                {'chart_data': chart_data, 'summary_data': summary_data,
                 'form': form, 'date_range': (from_date, to_date)},
                context_instance=RequestContext(request))


@superuser_required
def event_source_summary_report(request, source):
    queryset = EventLog.objects.filter(source=source)
    form = EventsFilterForm(request.GET)
    if form.is_valid():
        queryset = form.process_filter(queryset)

    from_date, to_date = request_month_range(request)
    next_day = to_date+timedelta(days=1)
    queryset = queryset.filter(create_dt__gte=from_date, create_dt__lte=next_day)

    chart_data = queryset\
                .extra(select={'day': 'DATE(create_dt)'})\
                .values('day', 'event_id')\
                .annotate(count=Count('pk'))\
                .order_by('day', '-count')
    chart_data = day_bars(chart_data, from_date.year, from_date.month, 300, event_colors)

    summary_data = queryset\
                .values('event_id', 'description')\
                .annotate(count=Count('pk'))\
                .order_by('-count')
    event_colors(summary_data)

    return render_to_response(
                'reports/event_source_summary.html',
                {'chart_data': chart_data, 'summary_data': summary_data,
                 'form': form, 'date_range': (from_date, to_date),
                 'source': source},
                context_instance=RequestContext(request))


@superuser_required
def info(request):
    apps = site.get_registered_apps().all_apps
    logged_models = []
    for app in apps:
        if 'event_logs' in app:
            for model in app['event_logs'].keys():
                logs = app['event_logs'][model]
                log_list = []
                for log in logs.keys():
                    log_list.append({
                        'label': log.replace('_', ' '),
                        'id': logs[log][0],
                        'color':logs[log][1],
                    })
                logged_models.append({
                    'label': model.replace('_', ' '),
                    'event_logs':sorted(log_list, key=lambda x: x['label']),
                })
    for model in non_model_event_logs.keys():
        logs = non_model_event_logs[model]
        log_list = []
        for log in logs.keys():
            log_list.append({
                'label': log.replace('_', ' '),
                'id': logs[log][0],
                'color':logs[log][1],
                'event_logs': sorted(log_list, key=lambda x: x['label'])
            })
        logged_models.append({
            'label': model.replace('_', ' '),
            'event_logs':sorted(log_list, key=lambda x: x['label']),
        })
    return render_to_response('event_logs/info.html', {
        'logged_models':sorted(logged_models, key=lambda x: x['label']),
    }, context_instance=RequestContext(request))



from django.conf import settings
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from tendenci.core.payments.models import Payment
from tendenci.core.payments.authorizenet.utils import prepare_authorizenet_sim_form
from tendenci.apps.invoices.models import Invoice
from tendenci.core.base.http import Http403
from tendenci.core.base.utils import tcurrency
from tendenci.core.event_logs.models import EventLog

from tendenci.core.site_settings.utils import get_setting

def pay_online(request, invoice_id, guid="", template_name="payments/pay_online.html"):
    # check if they have the right to view the invoice
    invoice = get_object_or_404(Invoice, pk=invoice_id)
    if not invoice.allow_view_by(request.user, guid): raise Http403
    
    # tender the invoice
    if not invoice.is_tendered:
        invoice.tender(request.user)
        # log an event for invoice edit
        log_defaults = {
            'event_id' : 312000,
            'event_data': '%s (%d) edited by %s' % (invoice._meta.object_name, invoice.pk, request.user),
            'description': '%s edited' % invoice._meta.object_name,
            'user': request.user,
            'request': request,
            'instance': invoice,
        }
        EventLog.objects.log(**log_defaults)  
      
    # generate the payment
    payment = Payment()
    
    boo = payment.payments_pop_by_invoice_user(request.user, invoice, guid)
    # log an event for payment add
    log_defaults = {
        'event_id' : 281000,
        'event_data': '%s (%d) added by %s' % (payment._meta.object_name, payment.pk, request.user),
        'description': '%s added' % payment._meta.object_name,
        'user': request.user,
        'request': request,
        'instance': payment,
    }
    EventLog.objects.log(**log_defaults)
    
    # post payment form to gateway and redirect to the vendor so customer can pay from there
    if boo:
        merchant_account = (get_setting("site", "global", "merchantaccount")).lower()
        
        if merchant_account == 'stripe':
            return HttpResponseRedirect(reverse('stripe.payonline', args=[payment.id]))    
        else:

            if merchant_account == "authorizenet":
                form = prepare_authorizenet_sim_form(request, payment)
                post_url = settings.AUTHNET_POST_URL
            elif merchant_account == 'firstdata':
                from tendenci.core.payments.firstdata.utils import prepare_firstdata_form
                form = prepare_firstdata_form(request, payment)
                post_url = settings.FIRSTDATA_POST_URL
            elif merchant_account == 'paypalpayflowlink':
                from tendenci.core.payments.payflowlink.utils import prepare_payflowlink_form
                form = prepare_payflowlink_form(request, payment)
                post_url = settings.PAYFLOWLINK_POST_URL
            elif merchant_account == 'paypal':
                from tendenci.core.payments.paypal.utils import prepare_paypal_form
                form = prepare_paypal_form(request, payment)
                post_url = settings.PAYPAL_POST_URL
            else:   # more vendors 
                form = None
                post_url = ""
    else:
        form = None
        post_url = ""
    return render_to_response(template_name, 
                              {'form':form, 'post_url':post_url}, 
                              context_instance=RequestContext(request))
    
def view(request, id, guid=None, template_name="payments/view.html"):
    payment = get_object_or_404(Payment, pk=id)

    if not payment.allow_view_by(request.user, guid): raise Http403
    #payment.amount = tcurrency(payment.amount)
    
    return render_to_response(template_name, {'payment':payment}, 
        context_instance=RequestContext(request))
    
def receipt(request, id, guid, template_name='payments/receipt.html'):
    payment = get_object_or_404(Payment, pk=id)
    if payment.guid <> guid:
        raise Http403
        
    return render_to_response(template_name,{'payment':payment},
                              context_instance=RequestContext(request))

        





from django.conf import settings
from django.core.paginator import Paginator, InvalidPage
from django.http import Http404
from django.shortcuts import render_to_response
from django.template import RequestContext

from tendenci.apps.search.forms import ModelSearchForm

RESULTS_PER_PAGE = getattr(settings, 'HAYSTACK_SEARCH_RESULTS_PER_PAGE', 20)

def open_search(request, template_name="search/open_search_xml.html"):
    return render_to_response(
        template_name, 
        mimetype="application/opensearchdescription+xml",
        context_instance=RequestContext(request)
    )
    

class SearchView(object):
    template = 'search/search.html'
    extra_context = {}
    query = ''
    results = []
    request = None
    form = None
    
    def __init__(self, template=None, load_all=False, form_class=ModelSearchForm, searchqueryset=None, context_class=RequestContext):
        self.load_all = load_all
        self.form_class = form_class
        self.context_class = context_class
        self.searchqueryset = searchqueryset
        self.user = None
        if template:
            self.template = template

    def __call__(self, request):
        """
        Generates the actual response to the search.
        
        Relies on internal, overridable methods to construct the response.
        """
        self.request = request
        self.user = self.request.user # to check for permissions
        
        self.form = self.build_form()
        self.query = self.get_query()
        self.results = self.get_results()
        
        return self.create_response()
    
    def build_form(self):
        """
        Instantiates the form the class should use to process the search query.
        """
        if self.searchqueryset is None:
            return self.form_class(self.request.GET, load_all=self.load_all, user=self.user)
        
        return self.form_class(self.request.GET, searchqueryset=self.searchqueryset, load_all=self.load_all, user=self.user)
    
    def get_query(self):
        """
        Returns the query provided by the user.
        
        Returns an empty string if the query is invalid.
        """
        if self.form.is_valid():
            return self.form.cleaned_data['q']
        
        return ''
    
    def get_results(self):
        """
        Fetches the results via the form.
        
        Returns an empty list if there's no query to search with.
        """
        if self.query:
            return self.form.search()
        
        return self.form.search()
    
    def build_page(self):
        """
        Paginates the results appropriately.
        
        In case someone does not want to use Django's built-in pagination, it
        should be a simple matter to override this method to do what they would
        like.
        """
        paginator = Paginator(self.results, RESULTS_PER_PAGE)
        
        try:
            page = paginator.page(self.request.GET.get('page', 1))
        except InvalidPage:
            raise Http404
        
        return (paginator, page)

    def extra_context(self):
        """
        Allows the addition of more context variables as needed.
        
        Must return a dictionary.
        """
        # TODO: Add the selected models so they can be shown in the H1
        return {}
    
    def create_response(self):
        """
        Generates the actual HttpResponse to send back to the user.
        """
        (paginator, page) = self.build_page()
        
        context = {
            'query': self.query,
            'form': self.form,
            'page': page,
            'paginator': paginator,
        }
        context.update(self.extra_context())
        
        return render_to_response(self.template, context, context_instance=self.context_class(self.request))


class FacetedSearchView(SearchView):
    def __name__(self):
        return "FacetedSearchView"
    
    def extra_context(self):
        extra = super(FacetedSearchView, self).extra_context()
        
        if self.results == []:
            extra['facets'] = self.form.search().facet_counts()
        else:
            extra['facets'] = self.results.facet_counts()
        
        return extra


def basic_search(request, template='search/search.html', load_all=True, form_class=ModelSearchForm, searchqueryset=None, context_class=RequestContext, extra_context=None):
    """
    A more traditional view that also demonstrate an alternative
    way to use Haystack.
    
    Useful as an example of for basing heavily custom views off of.
    
    Also has the benefit of thread-safety, which the ``SearchView`` class may
    not be.
    
    Template:: ``search/search.html``
    Context::
        * form
          An instance of the ``form_class``. (default: ``ModelSearchForm``)
        * page
          The current page of search results.
        * paginator
          A paginator instance for the results.
        * query
          The query received by the form.
    """
    query = ''
    results = []
    
    if request.GET.get('q'):
        form = form_class(request.GET, searchqueryset=searchqueryset, load_all=load_all)
        
        if form.is_valid():
            query = form.cleaned_data['q']
            results = form.search()
    else:
        form = form_class(searchqueryset=searchqueryset, load_all=load_all)
    
    paginator = Paginator(results, RESULTS_PER_PAGE)
    
    try:
        page = paginator.page(int(request.GET.get('page', 1)))
    except InvalidPage:
        raise Http404("No such page of results!")
    
    context = {
        'form': form,
        'page': page,
        'paginator': paginator,
        'query': query,
    }
    
    if extra_context:
        context.update(extra_context)
    
    return render_to_response(template, context, context_instance=context_class(request))



from haystack import indexes
from haystack import site

from django.db.models import signals

from tendenci.apps.invoices.models import Invoice
from tendenci.apps.search.indexes import CustomSearchIndex

class InvoiceIndex(CustomSearchIndex):
    text = indexes.CharField(document=True, use_template=True)
    object_type = indexes.CharField(model_attr='object_type', null=True)
    object_id = indexes.IntegerField(model_attr='object_id', null=True)
    bill_to = indexes.CharField(model_attr='bill_to')
    bill_to_first_name = indexes.CharField(model_attr='bill_to_first_name', default='')
    bill_to_last_name = indexes.CharField(model_attr='bill_to_last_name', default='')
    bill_to_company = indexes.CharField(model_attr='bill_to_company', default='')
    bill_to_address = indexes.CharField(model_attr='bill_to_address', default='')
    bill_to_city = indexes.CharField(model_attr='bill_to_city', default='')
    bill_to_state = indexes.CharField(model_attr='bill_to_state', default='')
    bill_to_zip_code = indexes.CharField(model_attr='bill_to_zip_code', default='')
    bill_to_country = indexes.CharField(model_attr='bill_to_country', default='')
    bill_to_phone = indexes.CharField(model_attr='bill_to_phone', default='')
    bill_to_email = indexes.CharField(model_attr='bill_to_email', default='')
    total = indexes.FloatField(model_attr='total')
    balance = indexes.FloatField(model_attr='balance')

    create_dt = indexes.DateTimeField(model_attr='create_dt')
    creator = indexes.CharField(model_attr='creator', null=True)
    creator_username = indexes.CharField(model_attr='creator_username', default='')
    owner = indexes.CharField(model_attr='owner', null=True)
    owner_username = indexes.CharField(model_attr='owner_username', default='')
    status = indexes.BooleanField(model_attr='status')
    status_detail = indexes.CharField(model_attr='status_detail')

    # PK: needed for exclude list_tags
    primary_key = indexes.CharField(model_attr='pk')

    def get_updated_field(self):
        return 'update_dt'

    def prepare_object_type(self, obj):
        myobj = obj.get_object()
        if myobj:
            return myobj._meta.verbose_name
        return obj.object_type

site.register(Invoice, InvoiceIndex)



from django import forms
from django.utils.encoding import force_unicode

class Output(forms.Widget):
    """
    Base class for all <output> widgets (e.g. titles and paragraphs).
    These are fake-fields; they do not take input.
    """

    def _format_value(self, value):
        if self.is_localized:
            return formats.localize_input(value)
        return value

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
        return force_unicode(self._format_value(value))
        
class Header(Output):
    """
    Outputs text.  Using class name to identify the type
    of text that is being output.
    """

class Description(Output):
    """
    Outputs text.  Using class name to identify the type
    of text that is being output.
    """



from django.conf import settings
from django.utils.translation import ugettext_noop as _
from django.db.models.signals import post_syncdb

# The post_syncdb signal is not being called because
# this profile application has migration files via
# the schemamigration command.
# TODO: The profiles application must use startmigration
# command in order to continue using the post_syncdb signal
# TODO: Or we can create a migration file for this function,
# as suggested by the site.
# http://south.aeracode.org/docs/commands.html?highlight=post_syncdb#initial-data-and-post-syncdb
if "tendenci.apps.notifications" in settings.INSTALLED_APPS:
    from tendenci.apps.notifications import models as notification

    def create_notice_types(app, created_models, verbosity, **kwargs):
        notification.create_notice_type("user_added", _("User Added"), _("A user has been added."))
        notification.create_notice_type("user_edited", _("User Edited"), _("A user has been edited."))
        notification.create_notice_type("user_deleted", _("User Deleted"), _("A user has been deleted"))

    post_syncdb.connect(create_notice_types, sender=notification)
else:
    print "Profiles: Skipping creation of NoticeTypes as notification app not found"



from django.contrib import admin
from django.core.urlresolvers import reverse

from tendenci.apps.notifications.models import (NoticeType, NoticeSetting, Notice,
    ObservedItem, NoticeEmail)


class NoticeTypeAdmin(admin.ModelAdmin):
    list_display = ('label', 'display', 'description', 'default')

    def has_add_permission(self, request):
        return False

    def has_change_permission(self, request, obj=None):
        return obj is None

    def __init__(self, *args, **kwargs):
        super(NoticeTypeAdmin, self).__init__(*args, **kwargs)
        self.list_display_links = (None, )


class NoticeSettingAdmin(admin.ModelAdmin):
    list_display = ('id', 'user', 'notice_type', 'medium', 'send')


class NoticeAdmin(admin.ModelAdmin):
    list_display = ('message', 'user', 'notice_type', 'added', 'unseen', 'archived')


class NoticeEmailAdmin(admin.ModelAdmin):
    list_display = ('preview_email', 'date_sent')
    actions = ['resend']

    def has_add_permission(self, request):
        return False

    def has_view_permission(self, request, obj=None):
        return False

    def preview_email(self, obj):
        return '<a href="%s">%s</a>' % \
            (reverse('notification_email', args=[obj.guid]), obj.title)
    preview_email.allow_tags = True
    preview_email.short_description = 'Preview Email'

    def resend(self, request, queryset):
        for q in queryset:
            print q.resend()
    resend.short_description = "Resend the selected emails"

    def has_change_permission(self, request, obj=None):
        return obj is None

    def __init__(self, *args, **kwargs):
        super(NoticeEmailAdmin, self).__init__(*args, **kwargs)
        self.list_display_links = (None, )

# admin.site.register(NoticeType, NoticeTypeAdmin)
# admin.site.register(NoticeSetting, NoticeSettingAdmin)
# admin.site.register(Notice, NoticeAdmin)
# admin.site.register(ObservedItem)
admin.site.register(NoticeEmail, NoticeEmailAdmin)



from django.template import Node, Library, TemplateSyntaxError, Variable

from tendenci.core.base.template_tags import ListNode, parse_tag_kwargs
from tendenci.addons.news.models import News

register = Library()


@register.inclusion_tag("news/options.html", takes_context=True)
def news_options(context, user, news):
    context.update({
        "opt_object": news,
        "user": user
    })
    return context


@register.inclusion_tag("news/nav.html", takes_context=True)
def news_nav(context, user, news=None):
    context.update({
        "nav_object": news,
        "user": user
    })
    return context


@register.inclusion_tag("news/search-form.html", takes_context=True)
def news_search(context):
    return context


class ListNewsNode(ListNode):
    model = News
    perms = 'news.view_news'


@register.tag
def list_news(parser, token):
    """
    Used to pull a list of :model:`news.News` items.

    Usage::

        {% list_news as [varname] [options] %}

    Be sure the [varname] has a specific name like ``news_sidebar`` or 
    ``news_list``. Options can be used as [option]=[value]. Wrap text values
    in quotes like ``tags="cool"``. Options include:
    
        ``limit``
           The number of items that are shown. **Default: 3**
        ``order``
           The order of the items. **Default: Latest Release Date**
        ``user``
           Specify a user to only show public items to all. **Default: Viewing user**
        ``query``
           The text to search for items. Will not affect order.
        ``tags``
           The tags required on items to be included.
        ``random``
           Use this with a value of true to randomize the items included.

    Example::

        {% list_news as news_list limit=5 tags="cool" %}
        {% for news_item in news_list %}
            {{ news_item.headline }}
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    context_var = bits[2]

    if len(bits) < 3:
        message = "'%s' tag requires more than 3" % bits[0]
        raise TemplateSyntaxError(message)

    if bits[1] != "as":
        message = "'%s' second argument must be 'as" % bits[0]
        raise TemplateSyntaxError(message)

    kwargs = parse_tag_kwargs(bits)

    if 'order' not in kwargs:
        kwargs['order'] = '-release_dt'

    return ListNewsNode(context_var, *args, **kwargs)



from django import forms
from django.utils.translation import ugettext_lazy as _
from tendenci.apps.user_groups.models import Group
from tendenci.core.imports.utils import get_header_list_from_content
from tendenci.core.imports.models import Import

KEY_CHOICES = (('email','Email'),
               ('first_name,last_name,email','First Name and Last Name and Email'),
               ('first_name,last_name,phone','First Name and Last Name and Phone'),
               ('first_name,last_name,company','First Name and Last Name and Company'),
               ('username','Username'),)


class UserImportForm(forms.Form):
    file  = forms.FileField(widget=forms.FileInput(attrs={'size': 35}))
    interactive = forms.CharField(widget=forms.RadioSelect(choices=((1,'Interactive'),
                                                          (0,'Not Interactive (no login)'),)), initial=0,)
    override = forms.CharField(widget=forms.RadioSelect(choices=((0,'Blank Fields'),
                                                          (1,'All Fields (override)'),)), initial=0, )
    key = forms.ChoiceField(initial="email", choices=KEY_CHOICES)
    group = forms.ModelChoiceField(queryset=Group.objects.filter(status=True, 
                                                                 status_detail='active').order_by('name'),
                                                                 empty_label='Select One', required=False)
    clear_group_membership = forms.BooleanField(initial=0, required=False)

    def clean(self):
        # test if the file is missing any key
        if self.cleaned_data.has_key('key') and self.cleaned_data.has_key('file'):
            key_list = self.cleaned_data["key"].split(',')
            file = self.cleaned_data['file']
            file_content = file.read()
            fields = get_header_list_from_content(file_content, file.name)

            fields = [f.strip('"') for f in fields]

            missing_keys = []
            for key in key_list:
                if key not in fields:
                    missing_keys.append(key)
            
            if missing_keys:
                missing_keys = ','.join(missing_keys)
                raise forms.ValidationError(_("The uploaded file lacks the required field(s) as the identity for duplicates: %s." % missing_keys))
        return self.cleaned_data


class UserImportPreviewForm(forms.Form):
    interactive = forms.CharField(widget=forms.HiddenInput(), required=False)
    override = forms.CharField(widget=forms.HiddenInput(), required=False)
    key = forms.CharField(widget=forms.HiddenInput())
    group = forms.CharField(widget=forms.HiddenInput(), required=False)
    clear_group_membership = forms.CharField(widget=forms.HiddenInput(), required=False)


class ImportForm(forms.ModelForm):
    class Meta:
        model = Import
        fields = ('file',)



from django.conf.urls.defaults import patterns, url
from tendenci.addons.news.feeds import LatestEntriesFeed
from tendenci.core.site_settings.utils import get_setting

urlpath = get_setting('module', 'news', 'url')
if not urlpath:
    urlpath = "news"

urlpatterns = patterns('tendenci.addons.news.views',
    url(r'^%s/$' % urlpath, 'search', name="news"),
    url(r'^%s/search/$' % urlpath, 'search_redirect', name="news.search"),
    url(r'^%s/print-view/(?P<slug>[\w\-\/]+)/$' % urlpath, 'print_view', name="news.print_view"),
    url(r'^%s/add/$' % urlpath, 'add', name="news.add"),
    url(r'^%s/edit/(?P<id>\d+)/$' % urlpath, 'edit', name="news.edit"),
    url(r'^%s/edit/meta/(?P<id>\d+)/$' % urlpath, 'edit_meta', name="news.edit.meta"),
    url(r'^%s/delete/(?P<id>\d+)/$' % urlpath, 'delete', name="news.delete"),
    url(r'^%s/feed/$' % urlpath, LatestEntriesFeed(), name='news.feed'),
    url(r'^%s/export/$' % urlpath, 'export', name='news.export'),
    url(r'^%s/(?P<slug>[\w\-\/]+)/$' % urlpath, 'detail', name="news.detail"),
)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'MakePayment'
        db.create_table('make_payments_makepayment', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('guid', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'], null=True)),
            ('first_name', self.gf('django.db.models.fields.CharField')(max_length=100, null=True)),
            ('last_name', self.gf('django.db.models.fields.CharField')(max_length=100, null=True)),
            ('company', self.gf('django.db.models.fields.CharField')(default='', max_length=50, null=True, blank=True)),
            ('address', self.gf('django.db.models.fields.CharField')(default='', max_length=100, null=True, blank=True)),
            ('address2', self.gf('django.db.models.fields.CharField')(default='', max_length=100, null=True, blank=True)),
            ('city', self.gf('django.db.models.fields.CharField')(default='', max_length=50, null=True, blank=True)),
            ('state', self.gf('django.db.models.fields.CharField')(default='', max_length=50, null=True, blank=True)),
            ('zip_code', self.gf('django.db.models.fields.CharField')(default='', max_length=50, null=True, blank=True)),
            ('country', self.gf('django.db.models.fields.CharField')(default='', max_length=50, null=True, blank=True)),
            ('email', self.gf('django.db.models.fields.CharField')(default='', max_length=50, null=True)),
            ('phone', self.gf('django.db.models.fields.CharField')(default='', max_length=50, null=True, blank=True)),
            ('referral_source', self.gf('django.db.models.fields.CharField')(default='', max_length=200, null=True, blank=True)),
            ('comments', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('payment_amount', self.gf('django.db.models.fields.DecimalField')(max_digits=10, decimal_places=2)),
            ('payment_count', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('payment_method', self.gf('django.db.models.fields.CharField')(default='cc', max_length=50)),
            ('invoice_id', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('creator', self.gf('django.db.models.fields.related.ForeignKey')(related_name='make_payment_creator', null=True, to=orm['auth.User'])),
            ('creator_username', self.gf('django.db.models.fields.CharField')(max_length=50, null=True)),
            ('owner', self.gf('django.db.models.fields.related.ForeignKey')(related_name='make_payment_owner', null=True, to=orm['auth.User'])),
            ('owner_username', self.gf('django.db.models.fields.CharField')(max_length=50, null=True)),
            ('status_detail', self.gf('django.db.models.fields.CharField')(default='estimate', max_length=50)),
            ('status', self.gf('django.db.models.fields.BooleanField')(default=True)),
        ))
        db.send_create_signal('make_payments', ['MakePayment'])


    def backwards(self, orm):
        
        # Deleting model 'MakePayment'
        db.delete_table('make_payments_makepayment')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'make_payments.makepayment': {
            'Meta': {'object_name': 'MakePayment'},
            'address': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'address2': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'city': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'comments': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'company': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'country': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'make_payment_creator'", 'null': 'True', 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True'}),
            'email': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'null': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'invoice_id': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'make_payment_owner'", 'null': 'True', 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True'}),
            'payment_amount': ('django.db.models.fields.DecimalField', [], {'max_digits': '10', 'decimal_places': '2'}),
            'payment_count': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'payment_method': ('django.db.models.fields.CharField', [], {'default': "'cc'", 'max_length': '50'}),
            'phone': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'referral_source': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'state': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'estimate'", 'max_length': '50'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True'}),
            'zip_code': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '50', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['make_payments']



from django.contrib import admin


class SettingAdmin(admin.ModelAdmin):
    list_display = ['name','label','value','scope','scope_category']
    search_fields = ['name','label']
    list_filter = ['scope_category']
    
    fieldsets = (
        (None, {'fields': ('value',)}),
    )    
        
    def queryset(self, request):
        qs = super(SettingAdmin, self).queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(client_editable=True)
    
class SettingAdminDev(admin.ModelAdmin):
    list_display = ['name','label','value','scope','scope_category']
    search_fields = ['name','label']
    list_filter = ['scope_category']
    
# admin.site.register(Setting, SettingAdminDev)



from django.contrib.auth.models import User

from tastypie.authorization import Authorization
from tastypie.resources import ModelResource
from tastypie import fields

from tendenci.core.api_tasty.serializers import SafeSerializer
from tendenci.core.api_tasty.auth import DeveloperApiKeyAuthentication
from tendenci.core.api_tasty.users.resources import UserResource

class TendenciResource(ModelResource):
    owner = fields.ForeignKey(UserResource, 'owner')
    creator = fields.ForeignKey(UserResource, 'creator')
    
    class Meta:
        serializer = SafeSerializer()
        authorization = Authorization()
        authentication = DeveloperApiKeyAuthentication()
    



from haystack import indexes

from django.db.models import signals

from tendenci.apps.search.signals import save_unindexed_item

class CustomSearchIndex(indexes.SearchIndex):
    """
    A custom SearchIndex subclass that saves the objects to the UnindexedItem table
    (if not already added) for later processing and deletes objects immediately.
    
    This requires a script to run the management command "process_unindexed" in the 
    background to update index. 
    """
    def _setup_save(self, model):
        signals.post_save.connect(save_unindexed_item, sender=model, weak=False)
        
    def _teardown_save(self, model):
        signals.post_save.disconnect(save_unindexed_item, sender=model)

    def _setup_delete(self, obj):
        signals.post_delete.connect(self.remove_object, sender=obj)

    def _teardown_delete(self, obj):
        signals.post_delete.disconnect(self.remove_object, sender=obj)


from django.conf.urls.defaults import *
from tendenci.addons.help_files.feeds import LatestEntriesFeed
from tendenci.core.site_settings.utils import get_setting

urlpath = get_setting('module', 'help_files', 'url')
if not urlpath:
    urlpath = "help-files"

urlpatterns = patterns('tendenci.addons.help_files.views',
    url(r'^%s/$' % urlpath, 'index', name='help_files'),
    url(r'^%s/search/$' % urlpath, 'search', name='help_files.search'),
    url(r'^%s/feed/$' % urlpath, LatestEntriesFeed(), name='help_files.feed'),
    url(r'^%s/add/$' % urlpath, 'add', name='help_files.add'),
    url(r'^%s/export/$' % urlpath, 'export', name='help_files.export'),
    url(r'^%s/edit/(?P<id>\d+)/$' % urlpath, 'edit', name='help_files.edit'),
    url(r'^%s/topic/(?P<id>\d+)/$' % urlpath, 'topic', name='help_files.topic'),
    url(r'^%s/requests/$' % urlpath, 'requests', name='help_files.requests'),
    url(r'^%s/request/$' % urlpath, 'request_new', name='help_files.request'),
    url(r'^%s/(?P<slug>[\w\-\/]+)/$' % urlpath, 'detail', name='help_file.details'),
)



from django.conf.urls.defaults import patterns, include, url

urlpatterns = patterns('tendenci.apps.wp_importer.views',
    url(r'^$', 'index'),
    url(r'^detail/(?P<task_id>[-\w]+)/$', 'detail', name='detail'),
)



from datetime import datetime
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
#from django.conf import settings
#from django.core.urlresolvers import reverse
from django.contrib.admin.views.decorators import staff_member_required
from django.db.models import Sum
from django.contrib import messages
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.utils import simplejson
from django.http import HttpResponse, Http404

from tendenci.addons.recurring_payments.models import (RecurringPayment, 
                                       PaymentProfile, 
                                       PaymentTransaction,
                                       RecurringPaymentInvoice)
from tendenci.addons.recurring_payments.authnet.utils import get_test_mode
from tendenci.addons.recurring_payments.utils import (RecurringPaymentEmailNotices, 
                                      run_a_recurring_payment)
from tendenci.addons.recurring_payments.authnet.cim import CIMCustomerProfile

from tendenci.core.base.http import Http403
from tendenci.core.event_logs.models import EventLog
from tendenci.core.base.decorators import ssl_required
from tendenci.core.base.utils import tcurrency
from tendenci.core.site_settings.utils import get_setting

@ssl_required
@login_required
def view_account(request, recurring_payment_id, 
                          template_name="recurring_payments/index.html"):
    """View a recurring payment account.
    """
    rp = get_object_or_404(RecurringPayment, pk=recurring_payment_id)
    
    # only admin or user self can access this page
    if not request.user.profile.is_superuser and request.user.id <> rp.user.id:
        raise Http403
    
    paid_payment_transactions = PaymentTransaction.objects.filter(
                                                recurring_payment=rp,
                                                status=True
                                                                )
    if paid_payment_transactions:
        last_paid_payment_transaction = paid_payment_transactions[0]
    else:
        last_paid_payment_transaction = None
        
    failed_payment_transactions = PaymentTransaction.objects.filter(
                                                recurring_payment=rp,
                                                status=False
                                                                )
    if failed_payment_transactions:
        last_failed_payment_transaction = failed_payment_transactions[0]
    else:
        last_failed_payment_transaction = None
    
    display_failed_transaction = False    
    if last_failed_payment_transaction:
        if not last_paid_payment_transaction or \
            last_failed_payment_transaction.create_dt  \
            > last_paid_payment_transaction.create_dt:
            display_failed_transaction = True
            
    if not rp.trial_amount:
        rp.trial_amount = 0
        
    # rp_invoices
    rp_invoices = RecurringPaymentInvoice.objects.filter(
                                        recurring_payment=rp
                                        ).order_by('-billing_cycle_start_dt')
                                        
    # payment transactions
    payment_transactions = PaymentTransaction.objects.filter(
                                        recurring_payment=rp
                                        ).order_by('-create_dt')
                                        
    # get ready for the add/update payment method button
    test_mode = get_test_mode()
    is_active = (rp.status_detail == 'active')
    if is_active:
        rp.populate_payment_profile()
        payment_profiles = PaymentProfile.objects.filter(
                            customer_profile_id=rp.customer_profile_id, 
                            status=True, status_detail='active')
        if payment_profiles:
            payment_profile = payment_profiles[0]
        else:
            payment_profile = None
            
    else:
        payment_profile = None
    
    is_owner = False   
    if request.user.id == rp.user.id: is_owner = True
    
    num_accounts = RecurringPayment.objects.filter(user=rp.user).count()
    
    return render_to_response(template_name, {
                                              'rp': rp,
                                              'display_failed_transaction': display_failed_transaction,
                                              'last_paid_payment_transaction': last_paid_payment_transaction,
                                              'last_failed_payment_transaction': last_failed_payment_transaction,
                                              'rp_invoices': rp_invoices,
                                              'payment_transactions': payment_transactions,
                                              'payment_profile': payment_profile,
                                              'test_mode': test_mode,
                                              'is_active': is_active,
                                              'is_owner': is_owner,
                                              'num_accounts': num_accounts
                                              }, 
        context_instance=RequestContext(request))
    
@ssl_required
@login_required
def my_accounts(request, username=None, 
                        template_name="recurring_payments/my_accounts.html"):
    """View a person's all recurring payment accounts.
    """
    isadmin = request.user.profile.is_superuser
    
    if isadmin and username:
        u = get_object_or_404(User, username=username)
    else:
        u = request.user
        
    #rps = RecurringPayment.objects.filter(user=u).values_list('id', flat=True).order_by('-id')
    rps = RecurringPayment.objects.filter(user=u).order_by('status_detail', '-id')
    
    if not rps:
        if isadmin:
            return HttpResponseRedirect(reverse('recurring_payment.customers'))
        raise Http404
    
    if len(rps) == 1:
        # they have only 1 account, go directly to that account
        return  HttpResponseRedirect(reverse('recurring_payment.view_account', args=[(rps[0]).id]))
    
    is_owner = False   
    if request.user.id == u.id: is_owner = True
    
    return render_to_response(template_name, {'rps': rps,
                                              'is_owner': is_owner,
                                              'account_user': u
                                              }, 
        context_instance=RequestContext(request))
    
    
@staff_member_required
def run_now(request):
    """Run a recurring payment.
    """
    rp_id = request.POST.get('rp_id')
    rp = get_object_or_404(RecurringPayment, pk=rp_id)
    
    result_data = {}
    result_data['processed'] = 'false'
    result_data['reason'] = 'done'
    
    payment_profiles = PaymentProfile.objects.filter(
                        customer_profile_id=rp.customer_profile_id,
                        status=True,
                        status_detail='active'
                        ).order_by('-update_dt')
    if not payment_profiles:
        valid_cpp_ids, invalid_cpp_ids = rp.populate_payment_profile()
        #print valid_cpp_ids, invalid_cpp_ids
        
        if valid_cpp_ids:
            payment_profiles = PaymentProfile.objects.filter(
                           customer_profile_id=valid_cpp_ids[0])        
                        
    if not payment_profiles:
        result_data['reason'] = 'not setup'
    else:
        if rp.status_detail == 'active':
            num_processed = run_a_recurring_payment(rp)
            if num_processed:
                result_data['processed'] = 'true'
                result_data['reason'] = 'processed'
                # get total_paid and balance for this rp
                result_data['total_paid'] = str(rp.total_paid)
                result_data['balance'] = str(rp.get_outstanding_balance())
                
                # get total amount received for all rps
                d = RecurringPaymentInvoice.objects.filter(
                                            invoice__balance=0,
                                            ).aggregate(total_amount_received=Sum('invoice__total'))
                result_data['total_amount_received'] = d['total_amount_received']
                if not result_data['total_amount_received']:
                    result_data['total_amount_received'] = 0
                result_data['total_amount_received'] = tcurrency(result_data['total_amount_received'])
                               
        
    return HttpResponse(simplejson.dumps(result_data))
    
    
@staff_member_required
def customers(request, template_name="recurring_payments/customers.html"):
    """Display a list of recurring payment accounts.
    """
    query = request.GET.get('q', None)
    if get_setting('site', 'global', 'searchindex') and query:
        recurring_payments = RecurringPayment.objects.search(query)
    else:
        recurring_payments = RecurringPayment.objects.all()
    total_customers = RecurringPayment.objects.count()
    # get total amount received
    d = RecurringPaymentInvoice.objects.filter(
                                invoice__balance=0,
                                ).aggregate(total_amount_received=Sum('invoice__total'))
    total_amount_received = d['total_amount_received']
    if not total_amount_received:
        total_amount_received = 0
    # get total amount unpaid
    d = RecurringPaymentInvoice.objects.filter(
                                invoice__balance__gt=0,
                                ).aggregate(total_amount_unpaid=Sum('invoice__balance'))
    total_amount_unpaid = d['total_amount_unpaid']
    if not total_amount_unpaid:
        total_amount_unpaid = 0
    
    # get total amount past due
    d = RecurringPaymentInvoice.objects.filter(
                                invoice__balance__gt=0,
                                billing_dt__lte=datetime.now()
                                ).aggregate(total_amount_past_due=Sum('invoice__balance'))
    total_amount_past_due = d['total_amount_past_due']
    if not total_amount_past_due:
        total_amount_past_due = 0
    
    return render_to_response(template_name, {
                    'recurring_payments': recurring_payments,
                    'total_customers': total_customers,
                    'total_amount_received': total_amount_received,
                    'total_amount_unpaid': total_amount_unpaid,
                    'total_amount_past_due': total_amount_past_due,
                    'query': query
                                              }, 
        context_instance=RequestContext(request))

  
def transaction_receipt(request, rp_id, payment_transaction_id, rp_guid=None,
                        template_name="recurring_payments/transaction_receipt.html"):
    """Display a transaction receipt.
    """
    if request.user.is_authenticated():
        rp = get_object_or_404(RecurringPayment, pk=rp_id)
        # only admin or user self can access this page
        if not request.user.profile.is_superuser and request.user.id <> rp.user.id:
            raise Http403
    else:
        if not rp_guid: raise Http403
        rp = get_object_or_404(RecurringPayment, pk=rp_id, guid=rp_guid)
    
    payment_transaction = get_object_or_404(PaymentTransaction, 
                                            pk=payment_transaction_id,
                                            status=True)
    payment_profile = PaymentProfile.objects.filter(
                    payment_profile_id=payment_transaction.payment_profile_id)[0]
    invoice = payment_transaction.payment.invoice

    
    return render_to_response(template_name, {
                    'rp': rp,
                    'invoice': invoice,
                    'payment_transaction': payment_transaction,
                    'payment_profile': payment_profile
                                              }, 
        context_instance=RequestContext(request))
    
    
@login_required
def disable_account(request, rp_id, 
                          template_name="recurring_payments/disable.html"):
    """Disable a recurring payment account.
    """
    rp = get_object_or_404(RecurringPayment, pk=rp_id)
    
    # only admin or user self can access this page
    if not request.user.profile.is_superuser and request.user.id <> rp.user.id:
        raise Http403
    if request.method == "POST":
        if request.POST.get('cancel'):
            return HttpResponseRedirect(reverse('recurring_payment.view_account', args=[rp.id]))
        else:
            
            rp.status_detail = 'disabled'
            rp.save()
            
            log_description = '%s disabled' % rp._meta.object_name
            
            # delete the CIM account - only if there is no other rps sharing the same customer profile
            is_deleted = rp.delete_customer_profile()
            if is_deleted:
                log_description = "%s as well as its CIM account." % log_description
                
                rp.customer_profile_id = ''
                rp.save()
                    
            
            
            # send an email to admin
            rp_email_notice = RecurringPaymentEmailNotices()
            rp_email_notice.email_admins_account_disabled(rp, request.user)
            
            
            # log an event
            log_defaults = {
                        'event_id' : 1120500,
                        'event_data': '%s (%d) disabled by %s' % (rp._meta.object_name, rp.pk, request.user),
                        'description': log_description,
                        'user': request.user,
                        'request': request,
                        'instance': rp,
            }
            EventLog.objects.log(**log_defaults)
            
            messages.add_message(request, messages.SUCCESS, 'Successfully disabled %s' % rp)
                                                                         
            return HttpResponseRedirect(reverse('recurring_payment.view_account', args=[rp.id]))
        
        
    return render_to_response(template_name, {
                    'rp': rp}, 
        context_instance=RequestContext(request))



from django.conf.urls.defaults import url, patterns

urlpatterns = patterns('tendenci.addons.tendenci_guide.views',
    url(r'^guide/(?P<slug>[\w\-]+)/$', 'guide_page', name="tendenci_guide.guide_page"),
)




import datetime
from django import forms
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from tendenci.addons.recurring_payments.models import BILLING_PERIOD_CHOICES, DUE_SORE_CHOICES

class BillingDateSelectInput(forms.TextInput):
    def render(self, name, value, attrs=None):
        return mark_safe('%s day(s) after billing cycle end date' \
                         % super(BillingDateSelectInput, self).render(name, value, attrs))
        
class BillingDateSelectWidget(forms.MultiWidget):
    def __init__(self, attrs=None):
        self.attrs = attrs
        self.pos_d = {'num_days': (0, forms.TextInput()),
                      'due_sore':(1, forms.Select()),
                       }
        self.widgets = ()
        if self.pos_d:
            items = self.pos_d.values()
            items.sort()
            self.widgets = [item[1] for item in items]
            
        super(BillingDateSelectWidget, self).__init__(self.widgets, attrs)
    
    def render(self, name, value, attrs=None):
        if not isinstance(value, list):
            value = self.decompress(value)
            
        final_attrs = self.build_attrs(attrs)
        id_ = final_attrs.get('id', None)
        
        # num_days
        num_days_widget = self.pos_d['num_days'][1]
        num_days_widget.attrs = {'size':'8'}
        rendered_num_days = self.render_widget(num_days_widget, name, value, final_attrs, 
                                             self.pos_d['num_days'][0], id_)
    
        # billing cycle start or end dt
        due_sore_widget = self.pos_d['due_sore'][1]
        due_sore_widget.choices = DUE_SORE_CHOICES
        rendered_due_sore = self.render_widget(due_sore_widget, 
                                    name, value, final_attrs, self.pos_d['due_sore'][0], id_)
        
        output_html = """
                        <div id="billing_dt_select">
                            %s day(s) after billing cycle %s date
                        </div>
                      """ % (rendered_num_days,
                             rendered_due_sore
                             )
                      
        return mark_safe(output_html)
        
        
        
    def render_widget(self, widget, name, value, attrs, index=0, id=None):
        i = index
        id_ = id
        if value:
            try:
                widget_value = value[i]
            except IndexError:
                self.fields['billing_dt_select'].initial = None
        else:
            widget_value = None
        if id_:
            final_attrs = dict(attrs, id='%s_%s' % (id_, i))

        return widget.render(name+'_%s' %i, widget_value, final_attrs)
        
    
    def decompress(self, value):
        if value:
            return value.split(",")
        return None
    
    
class BillingCycleWidget(forms.MultiWidget):
    def __init__(self, attrs=None):
        self.attrs = attrs
        self.pos_d = {'billing_frequency': (0, forms.TextInput()),
                      'billing_period': (1, forms.Select()),
                       }
        self.widgets = ()
        if self.pos_d:
            items = self.pos_d.values()
            items.sort()
            self.widgets = [item[1] for item in items]
            
        super(BillingCycleWidget, self).__init__(self.widgets, attrs)
    
    def render(self, name, value, attrs=None):
        if not isinstance(value, list):
            value = self.decompress(value)
            
        final_attrs = self.build_attrs(attrs)
        id_ = final_attrs.get('id', None)
        
        # billing_frequency
        billing_frequency_widget = self.pos_d['billing_frequency'][1]
        billing_frequency_widget.attrs = {'size':'8'}
        rendered_billing_frequency = self.render_widget(billing_frequency_widget, name, value, final_attrs, 
                                             self.pos_d['billing_frequency'][0], id_)
        
        # billing_period
        billing_period_widget = self.pos_d['billing_period'][1]
        billing_period_widget.choices = BILLING_PERIOD_CHOICES
        rendered_billing_period = self.render_widget(billing_period_widget, 
                                    name, value, final_attrs, self.pos_d['billing_period'][0], id_)
        
        
        output_html = """
                        <div id="billing_cycle">
                            Every %s %s
                        </div>
                      """ % (rendered_billing_frequency,
                             rendered_billing_period
                             )
                      
        return mark_safe(output_html)
        
        
        
    def render_widget(self, widget, name, value, attrs, index=0, id=None):
        i = index
        id_ = id
        if value:
            try:
                widget_value = value[i]
            except IndexError:
                self.fields['billing_cycle'].initial = None
        else:
            widget_value = None
        if id_:
            final_attrs = dict(attrs, id='%s_%s' % (id_, i))

        return widget.render(name+'_%s' %i, widget_value, final_attrs)
        
    
    def decompress(self, value):
        if value:
            return value.split(",")
        return None


from haystack import indexes
from haystack import site

from tendenci.apps.profiles.models import Profile
from tendenci.core.perms.indexes import TendenciBaseSearchIndex


class ProfileIndex(TendenciBaseSearchIndex):
    user = indexes.CharField(model_attr='user', faceted=True)
    user_object = indexes.CharField(model_attr='user', faceted=True)
    display_name = indexes.CharField(model_attr='display_name')
    company = indexes.CharField(model_attr='company')
    address = indexes.CharField(model_attr='address')
    city = indexes.CharField(model_attr='city')
    state = indexes.CharField(model_attr='state')
    zipcode = indexes.CharField(model_attr='zipcode')
    country = indexes.CharField(model_attr='country')
    last_name = indexes.CharField(faceted=True)
    hide_in_search = indexes.BooleanField(model_attr='hide_in_search')
    
    def prepare_last_name(self, obj):
        return obj.user.last_name

    def prepare_user_object(self, obj):
        return obj.user.username

    def prepare_user(self, obj):
        return "%s, %s (%s)" % (
            obj.user.last_name,
            obj.user.first_name,
            obj.user.username
        )

    def index_queryset(self):
        return Profile.objects.all().order_by('user')

site.register(Profile, ProfileIndex)



from django.utils.translation import ugettext, ugettext_lazy as _
from tendenci.core.base.fields import EmailVerificationField

FIELD_MAX_LENGTH = 2000
LABEL_MAX_LENGTH = 255

FIELD_TYPE_CHOICES = (
    ("CharField", _("Text")),
    ('DecimalField', _('Number')),
    ("CharField/django.forms.Textarea", _("Paragraph Text")),
    ("BooleanField", _("Checkbox")),
    ("ChoiceField", _("Choose from a list")),
    ("MultipleChoiceField", _("Multi select")),
    ("EmailVerificationField", _("Email")),
    ("DateField/django.forms.extras.SelectDateWidget", _("Date")),
    ("DateTimeField", _("Date/time")),
    ("CharField/tendenci.apps.forms_builder.forms.widgets.Description", _("Description")),
    ("CharField/tendenci.apps.forms_builder.forms.widgets.Header", _("Section Heading")),
)

USER_FIELD_CHOICES = (
    ('first_name', _('First Name')),
    ('last_name', _('Last Name')),
    ('mail_name', _('Mailing Name')),
    ('address', _('Address')),
    ('city', _('City')),
    ('state', _('State')),
    ('zip', _('Zip Code')),
    ('country', _('Country')),
    ('phone', _('Phone')),
    ('email', _('Email')),
    ('position_title', _('Position Title')),
    ('company_name', _('Company Name')),
    ('meal_option', _('Meal Option')),
    ('comments', _('Comments')) )



from django.template import Library
register = Library()

@register.inclusion_tag("dashboard/nav.html", takes_context=True)
def dashboard_nav(context, user):
    context.update({
        "user": user
    })
    return context


# settings - jobspaymenttypes, jobsrequirespayment
from datetime import datetime
from django.contrib.contenttypes.models import ContentType
from tendenci.addons.jobs.models import Job, JobPricing
from tendenci.apps.invoices.models import Invoice
from tendenci.core.payments.models import Payment
from tendenci.core.site_settings.utils import get_setting

def get_payment_method_choices(user):
    if user.profile.is_superuser:
        return (('paid - check', 'User paid by check'),
                ('paid - cc', 'User paid by credit card'),
                ('Credit Card', 'Make online payment NOW'),)
    else:
        job_payment_types = get_setting('module', 'jobs', 'jobspaymenttypes')
        if job_payment_types:
            job_payment_types_list = job_payment_types.split(',')
            job_payment_types_list = [item.strip() for item in job_payment_types_list]
            
            return [(item, item) for item in job_payment_types_list]
        else:
            return ()
        
def get_job_unique_slug(slug):
    # check if this slug already exists
    jobs = Job.objects.filter(slug__istartswith=slug)

    if jobs:
        t_list = [j.slug[len(j.slug):] for j in jobs]
        num = 1
        while str(num) in t_list:
            num += 1
        slug = '%s-%s' % (slug, str(num))
        
    return slug
  
def job_set_inv_payment(user, job, pricing):
    if get_setting('module', 'jobs', 'jobsrequirespayment'):
        if not job.invoice:
            inv = Invoice()
            inv.object_type = ContentType.objects.get(app_label=job._meta.app_label, 
                                              model=job._meta.module_name)
            inv.object_id = job.id
            inv.title = "Job Add Invoice"
            inv.bill_to = job.contact_name
            first_name = ''
            last_name = ''
            if job.contact_name:
                name_list = job.contact_name.split(' ')
                if len(name_list) >= 2:
                    first_name = name_list[0]
                    last_name = ' '.join(name_list[1:])
            inv.bill_to_first_name = first_name
            inv.bill_to_last_name = last_name
            inv.bill_to_company = job.contact_company
            inv.bill_to_address = job.contact_address
            inv.bill_to_city = job.contact_city
            inv.bill_to_state = job.contact_state
            inv.bill_to_zip_code = job.contact_zip_code
            inv.bill_to_country = job.contact_country
            inv.bill_to_phone = job.contact_phone
            inv.bill_to_fax = job.contact_fax
            inv.bill_to_email = job.contact_email
            inv.ship_to = job.contact_name
            inv.ship_to_first_name = first_name
            inv.ship_to_last_name = last_name
            inv.ship_to_company = job.contact_company
            inv.ship_to_address = job.contact_address
            inv.ship_to_city = job.contact_city
            inv.ship_to_state = job.contact_state
            inv.ship_to_zip_code = job.contact_zip_code
            inv.ship_to_country = job.contact_country
            inv.ship_to_phone = job.contact_phone
            inv.ship_to_fax = job.contact_fax
            inv.ship_to_email =job.contact_email
            inv.terms = "Due on Receipt"
            inv.due_date = datetime.now()
            inv.ship_date = datetime.now()
            inv.message = 'Thank You.'
            inv.status = True
            
            inv.total = get_job_price(user, job, pricing)
            inv.subtotal = inv.total
            inv.balance = inv.total
            inv.estimate = 1
            inv.status_detail = 'estimate'
            inv.save(user)

            # tender the invoice
            inv.tender(user)

            # update job
            job.invoice = inv
            job.save()
            
            if user.profile.is_superuser:
                if job.payment_method in ['paid - cc', 'paid - check', 'paid - wire transfer']:
                    boo_inv = inv.tender(user) 
                    
                    # payment
                    payment = Payment()
                    boo = payment.payments_pop_by_invoice_user(user, inv, inv.guid)
                    payment.mark_as_paid()
                    payment.method = job.payment_method
                    payment.save(user)
                    
                    # this will make accounting entry
                    inv.make_payment(user, payment.amount)
                    
            
def get_job_price(user, job, pricing):
    if user.profile.is_member:
        if job.list_type == 'regular':
            return pricing.regular_price_member
        else:
            return pricing.premium_price_member
    else:
        if job.list_type == 'regular':
            return pricing.regular_price
        else:
            return pricing.premium_price
    
    
def pricing_choices(user):
    """
    Since the list type of a job cannot be determined without the job,
    Both regular and premium price will be included in the label.
    """
    choices = []
    pricings = JobPricing.objects.all()
    for pricing in pricings:
        if user.profile.is_member:
            prices = "%s/%s" % (pricing.regular_price_member, pricing.premium_price_member)
        else:
            prices = "%s/%s" % (pricing.regular_price, pricing.premium_price)
            
        label = "%s: %s Days for %s" % (pricing.get_title(), pricing.duration, prices)
        choices.append((pricing.pk, label))
    return choices



import re

from decimal import Decimal

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User, AnonymousUser

from captcha.fields import CaptchaField
from tendenci.apps.discounts.models import Discount
from tendenci.core.site_settings.utils import get_setting
from tendenci.addons.memberships.models import Membership

from tendenci.addons.events.models import RegConfPricing, PaymentMethod, Registrant

class RegistrationForm(forms.Form):
    """
    Registration form.
    Focuses on non-registrant specific details.
    """
    amount_for_admin = forms.DecimalField(decimal_places=2, required=False)
    discount = forms.CharField(label=_('Discount Code'), required=False)
    captcha = CaptchaField(label=_('Type the code below'))
    payment_method = forms.ModelChoiceField(empty_label=None, required=True,
        queryset=PaymentMethod.objects.none(), widget=forms.RadioSelect, initial=1)
    
    def __init__(self, event, user, *args, **kwargs):
        """
        event: instance of Event model
        user: request.user
        reg_count: used for discount validation (discounts have usage limits)
        """
        self.event = event
        self.user = user
        self.reg_count = kwargs.pop('reg_count', 0)
        
        super(RegistrationForm, self).__init__(*args, **kwargs)
        
        # no need for captcha if logged in
        if user.is_authenticated():
            self.fields.pop('captcha')
        
        # admin only price override field
        if not user.profile.is_superuser:
            self.fields.pop('amount_for_admin')
        
        reg_conf =  event.registration_configuration
        if reg_conf.can_pay_online:
            payment_methods = reg_conf.payment_method.all()
        else:
            payment_methods = reg_conf.payment_method.exclude(
                machine_name='credit card').order_by('pk')
        self.fields['payment_method'].queryset = payment_methods
        
    def get_user(self):
        return self.user
        
    def get_event(self):
        return self.event
   
    def clean_discount(self):
        """
        Returns the discount instance if it exists for a given code.
        Returns none if the code is blank.
        """
        code = self.cleaned_data['discount']
        if code:
            try:
                discount = Discount.objects.get(discount_code=self.cleaned_data['discount'])
                if discount.available_for(self.reg_count):
                    return discount
                else:
                    raise forms.ValidationError(_("Discount Code cannot be used for %s people." % self.reg_count))
            except Discount.objects.DoesNotExist:
                raise forms.ValidationError(_("This is not a valid Discount Code!"))
        return code

class RegistrantForm(forms.Form):
    """
    Each registrant form will have a hidden pricing field.
    Each registrant form will have a hidden reg_set field.
    The reg_set field will be used to group the registrant data and validate
    them as a whole.
    """
    first_name = forms.CharField(max_length=50)
    last_name = forms.CharField(max_length=50)
    email = forms.EmailField()
    memberid = forms.CharField(label=_("Member ID"), max_length=50, required=False)
    company_name = forms.CharField(max_length=100, required=False)
    phone = forms.CharField(max_length=20, required=False)
    
    def __init__(self, *args, **kwargs):
        self.pricings = kwargs.pop('pricings')
        self.form_index = kwargs.pop('form_index', None)
        
        super(RegistrantForm, self).__init__(*args, **kwargs)
        
        # make the fields in the subsequent forms as not required
        if self.form_index and self.form_index > 0:
            for key in self.fields.keys():
                self.fields[key].required = False
        
        # initialize pricing options and reg_set field
        self.fields['pricing'] = forms.ModelChoiceField(widget=forms.HiddenInput, queryset=self.pricings)
        
        allow_memberid = get_setting('module', 'events', 'memberidpricing')
        if not allow_memberid:
            self.fields.pop('memberid')
        
        # initialize internal variables
        self.price = Decimal('0.00')
        self.saved_data = {}
        
    def set_price(self, price):
        self.price = price
        
    def get_price(self):
        return self.price
        
    def get_form_label(self):
        return self.form_index + 1
        
    def get_user(self):
        """
        Gets the user from memberid or email.
        Return AnonymousUser if both are unavailable.
        """
        user = AnonymousUser()
        memberid = self.saved_data.get('memberid', None)
        email = self.saved_data.get('email', None)

        if memberid:  # memberid takes priority over email
            membership = Membership.objects.first(member_number=memberid)
            if hasattr(membership, 'user'):
                user = membership.user
        elif email:
            users = User.objects.filter(email=email)
            if users:
                user = users[0]

        return user

    def clean_first_name(self):
        data = self.cleaned_data['first_name']
        
        # detect markup
        markup_pattern = re.compile('<[^>]*?>', re.I and re.M)
        markup = markup_pattern.search(data)
        if markup:
            raise forms.ValidationError("Markup is not allowed in the name field")

        # detect URL and Email
        pattern_string = '\w\.(com|net|org|co|cc|ru|ca|ly|gov)$'
        pattern = re.compile(pattern_string, re.I and re.M)
        domain_extension = pattern.search(data)
        if domain_extension or "://" in data:
            raise forms.ValidationError("URL's and Emails are not allowed in the name field")
        
        data.strip()
        return data
        
    def clean_last_name(self):
        data = self.cleaned_data['last_name']
        
        # detect markup
        markup_pattern = re.compile('<[^>]*?>', re.I and re.M)
        markup = markup_pattern.search(data)
        if markup:
            raise forms.ValidationError("Markup is not allowed in the name field")

        # detect URL and Email
        pattern_string = '\w\.(com|net|org|co|cc|ru|ca|ly|gov)$'
        pattern = re.compile(pattern_string, re.I and re.M)
        domain_extension = pattern.search(data)
        if domain_extension or "://" in data:
            raise forms.ValidationError("URL's and Emails are not allowed in the name field")
        
        data.strip()
        return data
    
    def clean_email(self):
        data = self.cleaned_data['email']
        data.strip()
        return data
    
    def _clean_fields(self):
        for name, field in self.fields.items():
            # value_from_datadict() gets the data from the data dictionaries.
            # Each widget type knows how to retrieve its own data, because some
            # widgets split data over several HTML fields.
            value = field.widget.value_from_datadict(self.data, self.files, self.add_prefix(name))
            try:
                if isinstance(field, forms.FileField):
                    initial = self.initial.get(name, field.initial)
                    value = field.clean(value, initial)
                else:
                    value = field.clean(value)
                self.cleaned_data[name] = value
                if hasattr(self, 'clean_%s' % name):
                    value = getattr(self, 'clean_%s' % name)()
                    self.cleaned_data[name] = value
            except forms.ValidationError, e:
                self._errors[name] = self.error_class(e.messages)
                if name in self.cleaned_data:
                    del self.cleaned_data[name]
            # save invalid or valid data into saved_data
            self.saved_data[name] = value

    def clean(self):
        data = self.cleaned_data
        pricing = self.cleaned_data['pricing']
        user = self.get_user()
        if not (user.is_anonymous() or pricing.allow_anonymous):
            already_registered = Registrant.objects.filter(user=user)
            if already_registered:
                if not user.profile.is_superuser:
                    raise forms.ValidationError('%s is already registered for this event' % user)
        return data



import os

from django.core.urlresolvers import reverse
from django.utils.functional import lazy
from django.conf import settings
from django.conf.urls.defaults import patterns, include

from tendenci.core.site_settings.utils import check_setting, get_setting

lazy_reverse = lazy(reverse, str)


class RegisteredApps(object):
    """
    RegistredApps iterarable that represents all registered apps

    All registered apps exist in the objects iterable and are also
    categorized by core/plugins.

    apps = site.get_registered_apps()

    core_apps = registered_apps.core
    addon_apps = registered_apps.addon
    people_apps = registered_apps.people
    """
    def __init__(self, apps, build_from_cache=False):
        self.all_apps = []
        self.core = []
        self.addons = []
        self.people = []

        # append core and plugin apps to
        # individual lists
        if not build_from_cache:
            for model, registry in apps.items():

                setting_tuple = (
                    'module',
                    registry.fields['model']._meta.app_label,
                    'enabled',
                )

                # enabled / has settings
                if check_setting(*setting_tuple):
                    registry.fields['enabled'] = get_setting(*setting_tuple)
                    registry.fields['has_settings'] = True
                else:
                    registry.fields['enabled'] = True
                    registry.fields['has_settings'] = False

                if not 'settings' in registry.fields['url'].keys():
                    registry.fields['url'].update({
                        'settings': lazy_reverse('settings.index', args=[
                            'module',
                            registry.fields['model']._meta.app_label
                        ])
                    })

                if registry.fields['app_type'] == 'addon':
                    self.addons.append(registry.fields)

                if registry.fields['app_type'] == 'people':
                    self.people.append(registry.fields)

                if registry.fields['app_type'] == 'core':
                    self.core.append(registry.fields)

                # append all apps for main iterable
                self.all_apps.append(registry.fields)
        else:
            #since we can only cache the list of apps and not the RegisteredApps instance
            #we have to rebuild this object based on the list of apps from the cache.
            for app in apps:

                setting_tuple = (
                    'module',
                    app['model']._meta.app_label,
                    'enabled',
                )

                # enabled / has settings
                if check_setting(*setting_tuple):
                    app['enabled'] = get_setting(*setting_tuple)
                    app['has_settings'] = True
                else:
                    app['enabled'] = True
                    app['has_settings'] = False

                if not 'settings' in app['url'].keys():
                    app['url'].update({
                        'settings': lazy_reverse('settings.index', args=[
                            'module',
                            app['model']._meta.app_label
                        ])
                    })

                if app['app_type'] == 'addon':
                    self.addons.append(app)

                if app['app_type'] == 'people':
                    self.people.append(app)

                if app['app_type'] == 'core':
                    self.core.append(app)

                # append all apps for main iterable
                self.all_apps.append(app)

        # sort the applications alphabetically by
        # object representation
        key = lambda x: unicode(x)
        self.all_apps = sorted(self.all_apps, key=key)
        self.core = sorted(self.core, key=key)
        self.addons = sorted(self.addons, key=key)
        self.people = sorted(self.people, key=key)

    def __iter__(self):
        return iter(self.all_apps)

    def __len__(self):
        return len(self.all_apps)


def update_addons(installed_apps, addon_folder_path):
    # Append only enabled addons to the INSTALLED_APPS
    addons = get_addons(installed_apps, addon_folder_path)
    installed_addons = tuple([i for i in addons])
    installed_apps += installed_addons

    return installed_apps


def get_addons(installed_apps, addon_folder_path):
    """
    Grabs a list of apps that aren't in INSTALLED_APPS
    """
    new_addons = []

    custom_addons = sorted(custom_choices(addon_folder_path))
    for addon in custom_addons:
        addon_package = '.'.join(['addons', addon])
        try:
            __import__(addon_package)
            new_addons.append(addon_package)
        except:
            pass

    return new_addons


def custom_choices(addon_folder_path):
    """
    Returns a list of available addons in the tendenci-site wrapper app
    """
    for addon in os.listdir(addon_folder_path):
        if os.path.isdir(os.path.join(addon_folder_path, addon)):
            yield addon


def get_url_patterns():
    items = []
    addons = get_addons(settings.INSTALLED_APPS, settings.SITE_ADDONS_PATH)
    for addon in addons:
        try:
            __import__('.'.join([addon, 'urls']))
            items.append((r'', include('%s.urls' % addon,)))
        except:
            pass

    return patterns('', *items)
    pass



import re

from decimal import Decimal

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User, AnonymousUser

from captcha.fields import CaptchaField
from tendenci.apps.discounts.models import Discount
from tendenci.core.site_settings.utils import get_setting

from tendenci.addons.events.models import Addon, AddonOption, RegAddon, RegAddonOption

class RegAddonForm(forms.Form):
    """RegAddon form during registration.
    The choices for the addon will depend on the registrants.
    Before this form can be validated the registrant formset must to be
    validated first.
    The valid_addons kwarg is the list of addons that the registrants are allowed to use.
    A RegAddonForm will dynamically add choice fields depending on the
    number of options it has.
    """
    
    def __init__(self, *args, **kwargs):
        self.addons = kwargs.pop('addons')
        self.valid_addons = kwargs.pop('valid_addons', [])
        self.form_index = kwargs.pop('form_index', None)
        self.chosen_addon = None
        super(RegAddonForm, self).__init__(*args, **kwargs)
        
        # initialize addon options and reg_set field
        self.fields['addon'] = forms.ModelChoiceField(
            queryset=self.addons,
            widget=forms.TextInput(attrs={'class': 'addon-input'}))
        
        # dynamically create an option field for each addon
        for addon in self.addons:
            field_name = addon.field_name()
            self.fields[field_name] = forms.ModelChoiceField(
                label="Options", required=False, empty_label=None,
                queryset=addon.options.all(),
                widget=forms.HiddenInput(attrs={'class': 'option-hidden'}))
        
    def get_form_label(self):
        return self.form_index + 1
        
    def clean_addon(self):
        addon = self.cleaned_data['addon']
        self.chosen_addon = addon
        if addon not in self.valid_addons:
            raise forms.ValidationError(_('Addon is invalid for current set of registrants'))
        return addon
            
    def clean(self):
        """Validate the option fields for the selected addon only"""
        data = self.cleaned_data
        if 'addon' in data:
            addon = data['addon']
            option = data[addon.field_name()]
            if not option:
                raise forms.ValidationError(_('Option required for %s' % (addon.title)))

        return data
    
    def save(self, registration):
        if self.is_valid():
            data = self.cleaned_data
            addon = data['addon']
            regaddon = RegAddon.objects.create(
                registration=registration,
                addon=addon,
                amount=addon.price,
            )
            option = data[addon.field_name()]
            RegAddonOption.objects.create(
                option = option,
                regaddon = regaddon,
            )
            return regaddon
        return None



from django.db import models
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django.db.models import Manager

from tendenci.core.categories.utils import prep_category

class CategoryManager(Manager):
    def update(self, object, value, type):
        ct = ContentType.objects.get_for_model(object)
        object_id = object.pk

        # get the category
        category = self.get_or_create(name=prep_category(value))[0]

        cat_item_filters = {
            'content_type': ct,
            'object_id': object_id
        }

        # they can only be in one sub category or category
        if type == 'category':
            cat_item_filters.update({'parent__exact':None})
            cat_item = CategoryItem._default_manager.get_or_create(**cat_item_filters)[0]
            cat_item.category = category
            cat_item.save()
        else: # sub category
            cat_item_filters.update({'category__exact':None})
            cat_item = CategoryItem._default_manager.get_or_create(**cat_item_filters)[0]
            cat_item.parent = category
            cat_item.save()


    def remove(self, object, type):
        ct = ContentType.objects.get_for_model(object)
        object_id = object.pk     
 
        cat_item_filters = {
            'content_type': ct,
            'object_id': object_id
        }

        if type == 'category':
            cat_item_filters.update({'parent__exact':None})
        else:
            cat_item_filters.update({'category__exact':None})  

        try:
            cat_item = CategoryItem._default_manager.get(**cat_item_filters)
        except:
            cat_item = None

        if cat_item:
            cat_item.delete()

    def get_for_model(self, model, category=None):
        """
        Returns a 2-tuple with lists inside. The tuple
        contains a list of categories and sub_categories.
        """
        ct = ContentType.objects.get_for_model(model)
        filters = {'content_type':ct}

        cat_items = CategoryItem.objects.filter(**filters).select_related('category__name','parent__name')
        categories = []
        sub_categories = []
        for cat in cat_items:
            if cat.category and cat.category not in categories:
                categories.append(cat.category)
            elif cat.parent and cat.parent not in sub_categories:
                sub_categories.append(cat.parent)

        #categories = set([c.category for c in cat_items if c.category])
        #sub_categories = set([self.get(pk=c.parent.pk) for c in cat_items if c.parent])

        # find all cat_items using category (ct & object_id)
        # find all cat_items using objects found within cat_items
        # find all subcategories associated with that object
        # return category from list of cat_items returned

        if category: 
            filters['category'] = category

            cat_items = CategoryItem.objects.filter(**filters)

            cat_items2 = []
            for cat_item in cat_items:
                cat_items3 = CategoryItem.objects.filter(
                    content_type=cat_item.content_type,
                    object_id=cat_item.object_id,
                )
                for i in cat_items3:
                    cat_items2.append(i)

            sub_categories = set([self.get(pk=c.parent.pk) for c in cat_items2 if c.parent])

        categories = sorted(categories, key=lambda category: category.name)
        sub_categories = sorted(sub_categories, key=lambda sub_categories: sub_categories.name)

        return (categories, sub_categories)


    def get_for_object(self, object, type):
        ct = ContentType.objects.get_for_model(object)
        object_id = object.pk

        cat_item_filters = {
            'content_type': ct,
            'object_id': object_id 
        }

        categories = CategoryItem._default_manager.filter(**cat_item_filters)

        if not categories: return None

        if type == 'category':
            for cat in categories:
                if cat.category:
                    return cat.category
        else: #it's a sub category
            for cat in categories:
                if cat.parent_id > 0:
                    return self.get(pk=cat.parent_id)
        return None

class Category(models.Model):
    name = models.CharField(max_length=255, db_index=True, unique=True)

    objects = CategoryManager()

    def __unicode__(self):
        return self.name

class CategoryItem(models.Model):
    content_type = models.ForeignKey(ContentType, db_index=True)
    object_id = models.PositiveIntegerField()
    category = models.ForeignKey(Category, related_name='%(class)s_category',null=True,blank=True)
    parent = models.ForeignKey(Category, related_name='%(class)s_parent', null=True,blank=True)
    object = generic.GenericForeignKey('content_type', 'object_id')
    
    def __unicode__(self):
        if self.category:
            return self.category.name
        return ""



import uuid

from timezones.fields import TimeZoneField

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib.contenttypes import generic

from tendenci.core.perms.models import TendenciBaseModel
from tendenci.core.perms.object_perms import ObjectPermission
from tendenci.apps.profiles.managers import ProfileManager, ProfileActiveManager

from tendenci.libs.abstracts.models import Person, Identity, Address


# class Profile(Identity, Address):
#     pass


class Profile(Person):
    # relations
    guid = models.CharField(max_length=40)
    pl_id = models.IntegerField(default=1)
    historical_member_number = models.CharField(_('historical member number'), max_length=50, blank=True)

    # profile meta data
    salutation = models.CharField(_('salutation'), max_length=15, 
        blank=True, choices=(('Mr.', 'Mr.'),('Mrs.', 'Mrs.'),
            ('Ms.', 'Ms.'),('Miss', 'Miss'),('Dr.', 'Dr.'),('Prof.', 'Prof.'),('Hon.', 'Hon.'),))
    initials = models.CharField(_('initials'), max_length=50, blank=True)
    display_name = models.CharField(_('display name'), max_length=120, blank=True)
    mailing_name = models.CharField(_('mailing name'), max_length=120, blank=True)
    company = models.CharField(_('company'), max_length=100, blank=True)
    position_title = models.CharField(_('position title'), max_length=50, blank=True)
    position_assignment = models.CharField(_('position assignment'), max_length=50, blank=True)
    sex = models.CharField(_('sex'), max_length=50, blank=True, choices=(('male', u'Male'),('female', u'Female')))
    address_type = models.CharField(_('address type'), max_length=50, blank=True)
    phone2 = models.CharField(_('phone2'), max_length=50, blank=True)
    fax = models.CharField(_('fax'), max_length=50, blank=True)
    work_phone = models.CharField(_('work phone'), max_length=50, blank=True)
    home_phone = models.CharField(_('home phone'), max_length=50, blank=True)
    mobile_phone = models.CharField(_('mobile phone'), max_length=50, blank=True)
    # email in auth_user
    email2 = models.CharField(_('email2'), max_length=200,  blank=True)
    url2 = models.CharField(_('url2'), max_length=100, blank=True)
    dob = models.DateTimeField(_('date of birth'), null=True, blank=True)
    ssn = models.CharField(_('social security number'), max_length=50, blank=True)
    spouse = models.CharField(_('spouse'), max_length=50, blank=True)
    department = models.CharField(_('department'), max_length=50, blank=True)
    education = models.CharField(_('education'), max_length=100, blank=True)
    student = models.IntegerField(_('student'), null=True, blank=True)
    remember_login = models.BooleanField(_('remember login'))
    exported = models.BooleanField(_('exported'))
    direct_mail = models.BooleanField(_('direct mail'), default=False)
    notes = models.TextField(_('notes'), blank=True)
    admin_notes = models.TextField(_('admin notes'), blank=True)
    referral_source = models.CharField(_('referral source'), max_length=50, blank=True)
    hide_in_search = models.BooleanField(default=False)
    hide_address = models.BooleanField(default=False)
    hide_email = models.BooleanField(default=False)
    hide_phone = models.BooleanField(default=False)
    first_responder = models.BooleanField(_('first responder'), default=False)
    agreed_to_tos = models.BooleanField(_('agrees to tos'), default=False)
    original_username = models.CharField(max_length=50)

    objects = ProfileManager()
    actives = ProfileActiveManager()

    class Meta:
        permissions = (("view_profile", "Can view profile"),)
        verbose_name = "User"
        verbose_name_plural = "Users"

    def __unicode__(self):
        return self.user.username

    @models.permalink
    def get_absolute_url(self):
        return ('profile', [self.user.username])

    def _can_login(self):
        """
        Private function used to verify active statuses in
        user and profile records.
        """
        return all([self.user.is_active, self.status, self.status_detail == "active"])

    @property
    def is_active(self):
        return self._can_login()

    @property
    def is_member(self):
        if self.member_number and self._can_login():
            return True
        return False

    @property
    def is_staff(self):
        if self.is_superuser:
            return True
        return all([self._can_login(), self.user.is_staff])

    @property
    def is_superuser(self):
        return all([self._can_login(), self.user.is_superuser])

    def save(self, *args, **kwargs):
        if not self.id:
            self.guid = str(uuid.uuid1())

        # match allow_anonymous_view with opposite of hide_in_search
        if self.hide_in_search:
            self.allow_anonymous_view = False
        else:
            self.allow_anonymous_view = True

        super(Profile, self).save(*args, **kwargs)

        try:
            from tendenci.addons.campaign_monitor.utils import update_subscription
            if hasattr(self, 'old_email') and getattr(self, 'old_email') != self.user.email:
                update_subscription(self, self.old_email)
                del self.old_email
        except ImportError:
            pass

    # if this profile allows view by user2_compare
    def allow_view_by(self, user2_compare):
        boo = False

        if user2_compare.is_superuser:
            boo = True
        else:
            if user2_compare == self.user:
                boo = True
            else:
                if self.creator == user2_compare or self.owner == user2_compare:
                    if self.status == 1:
                        boo = True
                else:
                    if user2_compare.has_perm('profiles.view_profile', self):
                        boo = True

        return boo

    # if this profile allows edit by user2_compare
    def allow_edit_by(self, user2_compare):
        if user2_compare.is_superuser:
            return True
        else:
            if user2_compare == self.user:
                return True
            else:
                if self.creator == user2_compare or self.owner == user2_compare:
                    if self.status == 1:
                        return True
                else:
                    if user2_compare.has_perm('profiles.change_profile', self):
                        return True
        return False

    def get_groups(self):
        memberships = self.user.group_member.all()
        return [membership.group for membership in memberships]

    def refresh_member_number(self):
        memberships = self.user.memberships.active_strict(order_by='-pk')

        if memberships:
            self.member_number = memberships[0].member_number
        else:
            self.member_number = u''

        self.save()
        return self.member_number

    def roles(self):
        role_set = []

        if self.is_superuser:
            role_set.append('superuser')

        if self.is_staff:
            role_set.append('staff')

        if self.is_member:
            role_set.append('member')

        if self.is_active:
            role_set.append('user')

        return role_set or ['disabled']

    def highest_role(self):
        """
        The highest role will be returned.
        """
        roles = ['superuser', 'staff', 'member', 'user', 'disabled']
        for role in roles:
            if role in self.roles():
                return role




from tastypie.serializers import Serializer

class SafeSerializer(Serializer):
    """
    Removes the optional serializers from the available formats
    to avoid dependency errors.
    """
    formats = ['json', 'jsonp', 'html']
    content_types = {
        'json': 'application/json',
        'jsonp': 'text/javascript',
        'html': 'text/html',
    }



import re
import shutil
import os
import sys
from optparse import make_option, OptionParser

from django.core.exceptions import ImproperlyConfigured
from django.core.management.color import color_style
from django.utils.encoding import smart_str
from django.conf import settings

def copy_helper(style, app_or_project, app_cap_single, app_low_single, app_cap_plural, app_low_plural, ev_id, other_name=''):
    """
    Copies either a Django application layout template or a Django project
    layout template into the specified directory.

    """
    # style -- A color style object (see django.core.management.color).
    # app_or_project -- The string 'app' or 'project'.
    # name -- The name of the application or project.
    # directory -- The directory to which the layout template should be copied.
    # other_name -- When copying an application layout, this should be the name
    #               of the project.
    
    app_or_project = 'plugin'
    directory = 'plugins'
    if not re.search(r'^[_a-zA-Z]\w*$', app_low_plural): # If it's not a valid directory name.
        # Provide a smart error message, depending on the error.
        if not re.search(r'^[_a-zA-Z]', app_low_plural):
            message = 'make sure the name begins with a letter or underscore'
        else:
            message = 'use only numbers, letters and underscores'
        raise CommandError("%r is not a valid %s name. Please %s." % (app_low_plural, app_or_project, message))
    top_dir = os.path.join(directory, app_low_plural)
    try:
        os.mkdir(top_dir)
    except OSError, e:
        pass

    # Determine where the app or project templates are. Use
    # django.__path__[0] because we don't know into which directory
    # django has been installed.
    template_dir = os.path.join(settings.PROJECT_ROOT, 'plugins', 'plugin_template')

    for d, subdirs, files in os.walk(template_dir):
        relative_dir = d[len(template_dir)+1:]
        if relative_dir:
            print relative_dir
            os.mkdir(os.path.join(top_dir, relative_dir.replace('plugin_template', app_low_plural)))
        for subdir in subdirs[:]:
            print subdir
            if subdir.startswith('.'):
                subdirs.remove(subdir)
        for f in files:
            if not f.endswith(('.py','.html','.txt')):
                # Ignore .pyc, .pyo, .py.class etc, as they cause various
                # breakages.
                continue
            path_old = os.path.join(d, f)
            path_new = os.path.join(top_dir, relative_dir.replace('plugin_template', app_low_plural), f.replace('S_P_LOW', app_low_plural).replace('S_S_LOW', app_low_single))
            fp_old = open(path_old, 'r')
            fp_new = open(path_new, 'w')
            fp_new.write(fp_old.read().replace('S_S_LOW', app_low_single).replace('S_P_LOW', app_low_plural).replace('S_S_CAP', app_cap_single).replace('S_P_CAP', app_cap_plural).replace('EVID', ev_id))
            fp_old.close()
            fp_new.close()
            try:
                shutil.copymode(path_old, path_new)
                _make_writeable(path_new)
            except OSError:
                sys.stderr.write(style.NOTICE("Notice: Couldn't set permission bits on %s. You're probably using an uncommon filesystem setup. No problem.\n" % path_new))

def _make_writeable(filename):
    """
    Make sure that the file is writeable. Useful if our source is
    read-only.

    """
    import stat
    if sys.platform.startswith('java'):
        # On Jython there is no os.access()
        return
    if not os.access(filename, os.W_OK):
        st = os.stat(filename)
        new_permissions = stat.S_IMODE(st.st_mode) | stat.S_IWUSR
        os.chmod(filename, new_permissions)


def update_addons(installed_apps):
    # Append only enabled addons to the INSTALLED_APPS
    addons = get_addons(installed_apps)
    installed_addons = tuple([i for i in addons])
    installed_apps += installed_addons

    return installed_apps


def get_addons(installed_apps):
    """
    Grabs a list of apps that aren't in INSTALLED_APPS
    """
    new_addons = []
    custom_addons = sorted(custom_choices())
    for addon in custom_addons:
        addon_package = '.'.join(['addons', addon])
        new_addons.append(addon_package)

    return new_addons


def plugin_options():
    """
    Returns a string of the available themes in THEMES_DIR
    """
    custom_options = []
    custom_addons = sorted(custom_choices())
    for addon in custom_addons:
        addon_package = '.'.join(['addons', addon])
        if addon_package not in settings.INSTALLED_APPS:
            custom_options.append((addon_package, addon.title().replace('_', ' ')))

    return custom_options


def custom_choices():
    """
    Returns a list of available addons in the tendenci-site wrapper app
    """
    for addon in os.listdir(settings.SITE_ADDONS_PATH):
        if os.path.isdir(os.path.join(settings.SITE_ADDONS_PATH, addon)):
            yield addon



"""
Linkedin OAuth support

No extra configurations are needed to make this work.
"""
import urlparse
from xml.etree import ElementTree

from tendenci.addons.social_auth.backends import ConsumerBasedOAuth, OAuthBackend


LINKEDIN_SERVER = 'linkedin.com'
LINKEDIN_REQUEST_TOKEN_URL = 'https://api.%s/uas/oauth/requestToken' % \
                                    LINKEDIN_SERVER
LINKEDIN_ACCESS_TOKEN_URL = 'https://api.%s/uas/oauth/accessToken' % \
                                    LINKEDIN_SERVER
LINKEDIN_AUTHORIZATION_URL = 'https://www.%s/uas/oauth/authenticate' % \
                                    LINKEDIN_SERVER
LINKEDIN_CHECK_AUTH = 'https://api.%s/v1/people/~' % LINKEDIN_SERVER


class LinkedinBackend(OAuthBackend):
    """Linkedin OAuth authentication backend"""
    name = 'linkedin'

    def get_user_details(self, response):
        """Return user details from Linkedin account"""
        return {
            'first_name': response['first-name'],
            'last_name': response['last-name'],
            'email': '',  # not supplied
        }


class LinkedinAuth(ConsumerBasedOAuth):
    """Linkedin OAuth authentication mechanism"""
    AUTHORIZATION_URL = LINKEDIN_AUTHORIZATION_URL
    REQUEST_TOKEN_URL = LINKEDIN_REQUEST_TOKEN_URL
    ACCESS_TOKEN_URL = LINKEDIN_ACCESS_TOKEN_URL
    SERVER_URL = 'api.%s' % LINKEDIN_SERVER
    AUTH_BACKEND = LinkedinBackend
    SETTINGS_KEY_NAME = 'LINKEDIN_CONSUMER_KEY'
    SETTINGS_SECRET_NAME = 'LINKEDIN_CONSUMER_SECRET'

    def user_data(self, access_token):
        """Return user data provided"""
        request = self.oauth_request(access_token, LINKEDIN_CHECK_AUTH)
        raw_xml = self.fetch_response(request)
        try:
            xml = ElementTree.fromstring(raw_xml)
            data = _xml_to_dict(xml)
            url = data['site-standard-profile-request']['url']
            url = url.replace('&amp;', '&')
            data['id'] = urlparse.parse_qs(url)['key'][0]

            return data
        except (xml.parsers.expat.ExpatError, KeyError, IndexError):
            return None

    @classmethod
    def enabled(cls):
        return True


def _xml_to_dict(xml):
    """Convert xml structure to dict"""
    data = {}
    for child in xml.getchildren():
        if child.getchildren():
            data[child.tag] = _xml_to_dict(child)
        else:
            data[child.tag] = child.text

    return data


# Backend definition
BACKENDS = {
    'linkedin': LinkedinAuth,
}



from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('tendenci.core.exports.views',
    url(r'^(?P<export_id>\d+)/$', 'status', name="export.status"),
    url(r'^(?P<export_id>\d+)/download/$', 'download', name="export.download"),
)




# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'UnindexedItem'
        db.create_table('search_unindexeditem', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('search', ['UnindexedItem'])


    def backwards(self, orm):
        
        # Deleting model 'UnindexedItem'
        db.delete_table('search_unindexeditem')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'search.unindexeditem': {
            'Meta': {'object_name': 'UnindexedItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {})
        }
    }

    complete_apps = ['search']



from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('tendenci.core.event_logs.views',
    url(r'^$', 'index', name="event_logs"),
    url(r'^reports/summary-historical/$', 'event_summary_historical_report', name='reports-events-summary-historical'),
    url(r'^reports/summary/$', 'event_summary_report', name='reports-events-summary'),
    url(r'^reports/summary/source/([^/]+)/$', 'event_source_summary_report', name='reports-events-source'),
    url(r'^reports/summary/application/(?P<application>[^/]+)/$', 'event_application_summary_report', name='reports-events-application'),
    url(r'^(?P<id>\d+)/$', 'index', name="event_log"),
    url(r'^search/$', 'search', name="event_log.search"),
    url(r'^print-view/(?P<id>\d+)/$', 'print_view', name="event_log.print_view"),
    url(r'^colored-image/(?P<color>\w+)/$', 'colored_image', name="event_log.colored_image"),
    url(r'^info/$', 'info', name="event_log.info"),
)



#!/usr/bin/env python

from __future__ import with_statement
from distutils.dir_util import copy_tree
from optparse import OptionParser
import os
from shutil import move
from uuid import uuid4

from django.utils.importlib import import_module


def create_project():
    """
    Copies the contents of the project_template directory to the
    current directory.

    The logic for this type of project build out came from Mezzanine
    https://github.com/stephenmcd/mezzanine/blob/master/mezzanine/bin/mezzanine_project.py
    """
    parser = OptionParser(usage="usage: %prog")
    project_path = os.path.join(os.getcwd())

    # Create the list of packages to build from - at this stage it
    # should only be one or two names, tendenci plus an alternate
    # package.
    packages = ["tendenci"]
    for package_name in packages:
        try:
            __import__(package_name)
        except ImportError:
            parser.error("Could not import package '%s'" % package_name)

    # Build the project up copying over the project_template from
    # each of the packages.
    for package_name in packages:
        package_path = os.path.dirname(os.path.abspath(import_module(package_name).__file__))
        copy_tree(os.path.join(package_path, "project_template"), project_path)
        move(os.path.join(project_path, ".env_example"),
             os.path.join(project_path, ".env"))

    # Update the local environment file with custom KEYs
    env_path = os.path.join(os.getcwd(), ".env")

    # Generate a unique SECREY_KEY for the project's setttings module.
    with open(env_path, "r") as f:
        data = f.read()
    with open(env_path, "w") as f:
        secret_key = "%s%s" % (uuid4(), uuid4())
        f.write(data.replace("your_unique_secret_key", secret_key))

    # Generate a unique SITE_SETTINGS_KEY for the project's setttings module.
    with open(env_path, "r") as f:
        data = f.read()
    with open(env_path, "w") as f:
        setting_key = "%s%s" % (uuid4(), uuid4())
        f.write(data.replace("your_tendenci_sites_settings_key", setting_key[:32]))

    # Clean up pyc files.
    for (root, dirs, files) in os.walk(project_path, False):
        for f in files:
            if f.endswith(".pyc"):
                os.remove(os.path.join(root, f))

if __name__ == "__main__":
    create_project()



#from haystack import indexes
#from haystack import site
#from tendenci.apps.entities.models import Entity
#from tendenci.core.perms.indexes import TendenciBaseSearchIndex
#
#
#class EntityIndex(TendenciBaseSearchIndex):
#    text = indexes.CharField(document=True, use_template=True)
#    entity_name = indexes.CharField(model_attr='entity_name')
#    entity_type = indexes.CharField(model_attr='entity_type')
#    entity_parent_id = indexes.IntegerField(model_attr='entity_parent_id')
#    contact_name = indexes.CharField(model_attr='contact_name')
#    phone = indexes.CharField(model_attr='phone')
#    fax = indexes.CharField(model_attr='fax')
#    email = indexes.CharField(model_attr='email')
#    website = indexes.CharField(model_attr='website')
#    summary = indexes.CharField(model_attr='summary')
#    notes = indexes.CharField(model_attr='notes')
#    admin_notes = indexes.CharField(model_attr='admin_notes')
#
#site.register(Entity, EntityIndex)



from django.db.models import signals
from haystack import indexes
from haystack import site
from tendenci.addons.recurring_payments.models import RecurringPayment
from tendenci.apps.search.indexes import CustomSearchIndex

class RecurringPaymentIndex(CustomSearchIndex):
    text = indexes.CharField(document=True, use_template=True)
    user = indexes.CharField(model_attr='user', faceted=True)
    user_object = indexes.CharField(model_attr='user', faceted=True)
    description = indexes.CharField(model_attr='description')
    payment_amount = indexes.FloatField(model_attr='payment_amount')   

    def get_updated_field(self):
        return 'update_dt'
    
    def prepare_user_object(self, obj):
        return obj.user.username

    def prepare_user(self, obj):
        return "%s" % (
            obj.user.get_full_name,
        )
        
    def index_queryset(self):
        return RecurringPayment.objects.all().order_by('user')
    

site.register(RecurringPayment, RecurringPaymentIndex)


import operator

from django import forms
from django.db import models
from django.utils.text import capfirst
from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.contrib.auth.models import User

from tendenci.core.site_settings.utils import get_setting
from tendenci.core.registry import site as registry_site
from tendenci.core.event_logs.models import EventLog
try:
    from tendenci.addons.corporate_memberships.models import CorporateMembership as CorpMemb
except:
    CorpMemb = None

import haystack
from haystack.query import SearchQuerySet

apps_not_to_search = [
    'discount',
    'donation',
    'file',
    'form',
    'box',
    'event_log',
    'invoice',
    'redirect',
    'user',
    'story',
]

registered_apps = registry_site.get_registered_apps()
registered_apps_names = [app['model']._meta.module_name for app in registered_apps \
                        if app['verbose_name'].lower() not in apps_not_to_search]
registered_apps_models = [app['model'] for app in registered_apps \
                         if app['verbose_name'].lower() not in apps_not_to_search]


def model_choices(site=None):
    if site is None:
        site = haystack.sites.site

    choices = []
    for m in site.get_indexed_models():
        if m._meta.module_name.lower() in registered_apps_names:
            choices.append(("%s.%s" % (m._meta.app_label, m._meta.module_name), 
                                capfirst(unicode(m._meta.verbose_name_plural))))
            
    return sorted(choices, key=lambda x: x[1])


class SearchForm(forms.Form):
    q = forms.CharField(required=False, label=_('Search'), max_length=255)
    
    def __init__(self, *args, **kwargs):
        self.searchqueryset = kwargs.get('searchqueryset', None)
        self.load_all = kwargs.get('load_all', False)
        self.user = kwargs.get('user', None)
        
        if self.searchqueryset is None:
            self.searchqueryset = SearchQuerySet()
        
        try:
            del(kwargs['searchqueryset'])
        except KeyError:
            pass
        
        try:
            del(kwargs['load_all'])
        except KeyError:
            pass

        try:
            del(kwargs['user'])
        except KeyError:
            pass
                
        super(SearchForm, self).__init__(*args, **kwargs)
    
    def search(self, order_by='newest'):
        self.clean()

        sqs = SearchQuerySet()
        user = self.user
        query = self.cleaned_data['q']

        # check to see if there is impersonation
        if hasattr(user,'impersonated_user'):
            if isinstance(user.impersonated_user, User):
                user = user.impersonated_user
                
        is_an_admin = user.profile.is_superuser
        
        # making a special case for corp memb because it needs to utilize two settings
        # (anonymoussearchcorporatemembers and membersearchcorporatemembers)
        if CorpMemb and self.cleaned_data['models'] == ["%s.%s" % (CorpMemb._meta.app_label, 
                                                                   CorpMemb._meta.module_name)]:
            filter_and, filter_or = CorpMemb.get_search_filter(user)
            q_obj = None
            if filter_and:
                q_obj = Q(**filter_and)
            if filter_or:
                q_obj_or = reduce(operator.or_, [Q(**{key: value}) for key, value in filter_or.items()])
                if q_obj:
                    q_obj = reduce(operator.and_, [q_obj, q_obj_or])
                else:
                    q_obj = q_obj_or
            if q_obj:
                sqs = sqs.filter(q_obj)
            
            if query:
                sqs = sqs.auto_query(sqs.query.clean(query)) 
        else:
            
            if query:
                sqs = sqs.auto_query(sqs.query.clean(query)) 
                if user:
                    if not is_an_admin:
                        if not user.is_anonymous():
                        # if b/w admin and anon
    
                            # (status+status_detail+(anon OR user)) OR (who_can_view__exact)
                            anon_query = Q(**{'allow_anonymous_view':True,})
                            user_query = Q(**{'allow_user_view':True,})
                            sec1_query = Q(**{
                                'status':1,
                                'status_detail':'active',
                            })
                            sec2_query = Q(**{
                                'owner__exact':user.username
                            })
                            query = reduce(operator.or_, [anon_query, user_query])
                            query = reduce(operator.and_, [sec1_query, query])
                            query = reduce(operator.or_, [query, sec2_query])
    
                            sqs = sqs.filter(query)
                        else:
                        # if anonymous
                            sqs = sqs.filter(status=True).filter(status_detail='active')
                            sqs = sqs.filter(allow_anonymous_view=True)
                else:
                    # if anonymous
                    sqs = sqs.filter(status=True).filter(status_detail='active')
                    sqs = sqs.filter(allow_anonymous_view=True)
            else:
                if user:
                    if is_an_admin:
                        sqs = sqs.all()
                    else:
                        if not user.is_anonymous():
                            # (status+status_detail+anon OR who_can_view__exact)
                            sec1_query = Q(**{
                                'status':1,
                                'status_detail':'active',
                                'allow_anonymous_view':True,
                            })
                            sec2_query = Q(**{
                                'owner__exact':user.username
                            })
                            query = reduce(operator.or_, [sec1_query, sec2_query])
                            sqs = sqs.filter(query)
                        else:
                            # if anonymous
                            sqs = sqs.filter(status=True).filter(status_detail='active')
                            sqs = sqs.filter(allow_anonymous_view=True)               
                else:
                    # if anonymous
                    sqs = sqs.filter(status=True).filter(status_detail='active')
                    sqs = sqs.filter(allow_anonymous_view=True)
            
            # for solr,
            # order_by can only called once so we have to do it here
            if order_by == 'newest':
                sqs = sqs.order_by('-create_dt')
            else:
                sqs = sqs.order_by('create_dt')
            
        if self.load_all:
            sqs = sqs.load_all()
        
        return sqs


class HighlightedSearchForm(SearchForm):
    def search(self):
        return super(HighlightedSearchForm, self).search().highlight()


class FacetedSearchForm(SearchForm):
    selected_facets = forms.CharField(required=False, widget=forms.HiddenInput)
    
    def search(self):
        sqs = super(FacetedSearchForm, self).search()
        
        if self.cleaned_data['selected_facets']:
            sqs = sqs.narrow(self.cleaned_data['selected_facets'])
        
        return sqs


class ModelSearchForm(SearchForm):
    SORT_CHOICES = (
        ('newest','Newest'),
        ('oldest', 'Oldest')
        #('most_viewed', 'Most Viewed')
    )
    sort_by = forms.ChoiceField(choices=SORT_CHOICES, required=False)
    
    def __init__(self, *args, **kwargs):
        super(ModelSearchForm, self).__init__(*args, **kwargs)

        # Check to see if users should be included in global search
        include_users = False

        if kwargs['user'].profile.is_superuser or get_setting('module', 'users', 'allowanonymoususersearchuser') \
        or (kwargs['user'].is_authenticated() and get_setting('module', 'users', 'allowusersearch')):
            include_users = True

        if include_users:
            for app in registered_apps:
                if app['verbose_name'].lower() == 'user':
                    registered_apps_models.append(app['model'])
                    registered_apps_names.append(app['model']._meta.module_name)
        else:
            for app in registered_apps:
                if app['verbose_name'].lower() == 'user':
                    try:
                        models_index = registered_apps_models.index(app['model'])
                        registered_apps_models.pop(models_index)
                        names_index = registered_apps_names.index(app['model']._meta.module_name)
                        registered_apps_names.pop(names_index)
                    except Exception as e:
                        pass

        self.models = registered_apps_models
        self.fields['models'] = forms.MultipleChoiceField(choices=model_choices(), required=False, label=_('Search In'), widget=forms.CheckboxSelectMultiple)

    def get_models(self):
        """Return an alphabetical list of model classes in the index."""
        search_models = self.models
        if self.cleaned_data['models']:
            search_models = []
            for model in self.cleaned_data['models']:
                class_model = models.get_model(*model.split('.'))
                #if class_model._meta.module_name in INCLUDED_APPS:
                search_models.append(class_model)
        return search_models
    
    def search(self):
        sqs = super(ModelSearchForm, self).search(order_by=self.cleaned_data['sort_by'])
        sqs = sqs.models(*self.get_models())
        if self.cleaned_data['sort_by'] == 'most_viewed':
            # we need to query for number of views
            for s in sqs:
                instance = s.object
                ct = ContentType.objects.get_for_model(instance)
                views = EventLog.objects.filter(
                            content_type=ct,
                            action__icontains='detail',
                            object_id=instance.pk).count()
                s.views = views
            # We need to do this unless we
            # we have a 'view count' field for each model
            sqs = sorted(sqs, key=lambda s: s.views, reverse=True) # no longer an sqs
        return sqs


class HighlightedModelSearchForm(ModelSearchForm):
    def search(self):
        return super(HighlightedModelSearchForm, self).search().highlight()


class FacetedModelSearchForm(ModelSearchForm):
    selected_facets = forms.CharField(required=False, widget=forms.HiddenInput)
    
    def search(self):
        sqs = super(FacetedModelSearchForm, self).search()
        
        if self.cleaned_data['selected_facets']:
            sqs = sqs.narrow(self.cleaned_data['selected_facets'])
        
        return sqs.models(*self.get_models())




import re
from django.conf import settings
from django.core.urlresolvers import reverse
from tendenci.core.site_settings.utils import get_setting

direct_response_fields = (
                        'response_code',
                        'response_subcode',
                        'response_reason_code',
                        'response_reason_text',
                        'auth_code',
                        'avs_code',
                        'trans_id',
                        'invoice_num',
                        'description',
                        'amount',
                        'method',
                        'trans_type',
                        'customer_id',
                        'first_name',
                        'last_name',
                        'company',
                        'address',
                        'city',
                        'state',
                        'zip',
                        'country',
                        'phone',
                        'fax',
                        'email',
                        'ship_to_first_name',
                        'ship_to_last_name',
                        'ship_to_company',
                        'ship_to_address',
                        'ship_to_city',
                        'ship_to_state',
                        'ship_to_zip',
                        'ship_to_country',
                        'tax',
                        'duty',
                        'freight',
                        'tax_exempt',
                        'purchase_order_number',
                        'md5_hash',
                        'card_code_response',
                        'cardholder_auth_verification',
                        'response',
                        'account_number',
                        'card_type',
                        'split_tender_id',
                        'requested_amount',
                        'balance_on_card',
                          )

def direct_response_dict(direct_response_str):
    """
    Return a dictionary from a direct response string. 
    """
#    direct_response_str = "1,1,1,This transaction has been" + \
#        "approved.,000000,Y,2000000001,INV000001,description of" + \
#        "transaction,10.95,CC,auth_capture,custId123,John,Doe,,123 Main" + \
#        "St.,Bellevue,WA,98004,USA,000-000-" + \
#        "0000,,mark@example.com,John,Doe,,123 Main St.,Bellevue,WA,98004,USA,1.00,0.00,2.00,FALSE,PONUM000001," + \
#        "D18EB6B211FE0BBF556B271FDA6F92EE,M,2,,,,,,,,,,,,,,,,,,,,,,,,,,,,"

    response_dict = {}
    max_length = len(direct_response_fields)
    # the x_delim_char we specified is || (in cim.py)
    direct_response_list = direct_response_str.split('||')
    for i, value in enumerate(direct_response_list):
        if i < max_length:
            response_dict[direct_response_fields[i]] = value
            
    return response_dict

def payment_update_from_response(payment, direct_response_str):
    """
    Update the payment entry with the direct response from payment gateway.
    """
    response_dict = direct_response_dict(direct_response_str)
    for key in response_dict.keys():
        if hasattr(payment, key):
            setattr(payment, key, response_dict[key])
            
    return payment


def to_camel_case(d):
    """
	Convert all keys in d dictionary from underscore_format to
	camelCaseFormat and return the new dict
	"""
    if type(d) is dict:
        to_upper = lambda match: match.group(1).upper()
        to_camel = lambda x: re.sub("_([a-z])", to_upper, x)
        
        return dict(map(lambda x: (to_camel(x[0]), x[1]), d.items()))
    return d

  
def get_test_mode():
    """Return test_mode (false/true) - to be used in js
    """
    test_mode = 'false'
    if hasattr(settings, 'AUTHNET_CIM_TEST_MODE') and  settings.AUTHNET_CIM_TEST_MODE:
        test_mode = 'true'
       
    return test_mode
    
def get_token(rp, CIMCustomerProfile, CIMHostedProfilePage, iframe_communicator_url='', is_secure=False):
    """Get the token from payment gateway for this customer (ex: customer_profile_id=4356210).
       Return token and gateway_error
    """
    gateway_error = False
    
    site_url = get_setting('site', 'global', 'siteurl')
    if is_secure:
        site_url = site_url.replace('http://', 'https://')
    if not iframe_communicator_url:
        iframe_communicator_url = '%s%s' % (
                                site_url, 
                                reverse('recurring_payment.authnet.iframe_communicator'))
    
    token = ""
    if not rp.customer_profile_id:
        # customer_profile is not available yet for this customer, create one now
        cp = CIMCustomerProfile()
        d = {'email': rp.user.email,
             'description': rp.description,
             'customer_id': str(rp.id)}
        success, response_d = cp.create(**d)
        if success:
            rp.customer_profile_id = response_d['customer_profile_id']
            rp.save()
        else:
            gateway_error = True
            # return since we can't continue without a profile id
            return token, gateway_error

    hosted_profile_page = CIMHostedProfilePage(rp.customer_profile_id)
    
    d = {'hosted_profile_settings': 
         {'hosted_profile_heading_bg_color': '#e0e0e0',     # the bg color of sections can be customized
         'hosted_profile_iFrame_communicator_url': '%s' % iframe_communicator_url}}
    success, response_d = hosted_profile_page.get(**d)
    #print success, response_d
    
    if not success:
        gateway_error = True
    else:
        token = response_d['token']
    
    return token, gateway_error



from django.conf import settings

mobile_agents = [
#    'iPad',  # Removed on 2012-07-11
    'iPhone',
    'iPod',
    'Android',
    'Opera Mini',
    'Blackberry',
    'Droid',
    'IEMobile',
    'EudoraWeb',
    'Fennec',
    'Minimo',
    'NetFront',
    'Polaris',
    'HTC_Dream',
    'HTC Hero',
    'HTC-ST7377',
    'Kindle',
    'LG-LX550',
    'LX265',
    'Nokia',
    'Palm',
    'MOT-V9mm',
    'SEC-SGHE900',
    'SAMSUNG-SGH-A867',
    'SymbianOS',
    'DoCoMo',
    'ZuneHD',
    'ReqwirelessWeb',
    'SEJ001',
    'SonyEricsson'
]

def user_agent(request):
    if 'HTTP_USER_AGENT' in request.META:
        return request.META['HTTP_USER_AGENT'].lower()
    return None

def is_mobile_cookie_on(request):
    cookiename =  getattr(settings, 'MOBILE_COOKIE_NAME', "tendenci_mobile")
    if cookiename in request.COOKIES:
        if request.COOKIES[cookiename] == "0":  
            return False
    return True

def is_mobile_browser(request):
    if request.user_agent:
        for ma in mobile_agents:
            if ma.lower() in request.user_agent:
                return True
    return False
    
def show_mobile(request):
    if not is_mobile_cookie_on(request):
        # no need to check any further, the user opt-out
        return False
    
    if is_mobile_browser(request):
        return True
    return False
    

class MobileMiddleware(object):
    def process_request(self, request):
        request.user_agent = user_agent(request)
        request.mobile_browser = is_mobile_browser(request)
        request.mobile = show_mobile(request)



#import os
import math
import traceback
#from datetime import datetime
from django.shortcuts import render_to_response, get_object_or_404
#from django.http import HttpResponse
from django.conf import settings
from django.template import RequestContext
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.views.decorators.csrf import csrf_exempt

import simplejson

from tendenci.core.payments.utils import payment_processing_object_updates
from tendenci.core.payments.utils import log_payment, send_payment_notice
from tendenci.core.payments.models import Payment
from forms import StripeCardForm, BillingInfoForm
import stripe
from utils import payment_update_stripe


@csrf_exempt
def pay_online(request, payment_id, template_name='payments/stripe/payonline.html'):
    payment = get_object_or_404(Payment, pk=payment_id) 
    form = StripeCardForm(request.POST or None)
    billing_info_form = BillingInfoForm(request.POST or None, instance=payment)
    if request.method == "POST":
        if form.is_valid():
            # get stripe token and make a payment immediately
            stripe.api_key = getattr(settings, 'STRIPE_SECRET_KEY', '')
            token = request.POST.get('stripe_token')
            
            if billing_info_form.is_valid():
                payment = billing_info_form.save()
            
            # create the charge on Stripe's servers - this will charge the user's card
            params = {
                       'amount': math.trunc(payment.amount * 100), # amount in cents, again
                       'currency': "usd", # currently, only 'usd' is supported
                       'card': token,
                       'description': payment.description
                      }
            
            try:
                charge_response = stripe.Charge.create(**params)
                # an example of response: https://api.stripe.com/v1/charges/ch_YjKFjLIItzRDv7
                #charge_response = simplejson.loads(charge)
            except:
                charge_response = traceback.format_exc()
                print 'error=', charge_response
            
            
            # update payment status and object
            if  payment.invoice.balance > 0:
                payment_update_stripe(request, charge_response, payment)
                payment_processing_object_updates(request, payment)
                
                # log an event
                log_payment(request, payment)
                
                # send payment recipients notification
                send_payment_notice(request, payment) 
            
            # redirect to thankyou
            return HttpResponseRedirect(reverse('stripe.thank_you', args=[payment.id]))
        
    return render_to_response(template_name, {'form': form, 
                                              'billing_info_form': billing_info_form,
                                              'payment': payment},
                              context_instance=RequestContext(request))

@csrf_exempt
def thank_you(request, payment_id, template_name='payments/receipt.html'):
    #payment, processed = stripe_thankyou_processing(request, dict(request.POST.items()))
    payment = get_object_or_404(Payment, pk=payment_id) 

    return render_to_response(template_name,{'payment':payment}, 
                              context_instance=RequestContext(request))


import os
import re
from datetime import datetime
from datetime import timedelta
from dateutil.relativedelta import relativedelta

from django.conf import settings
from django.template.defaultfilters import slugify
from django.core.files.storage import default_storage
from django.template.loader import get_template
from django.template import TemplateDoesNotExist

from tendenci.core.site_settings.utils import get_setting
from tendenci.core.theme.utils import get_theme_root

STOP_WORDS = ['able','about','across','after','all','almost','also','am',
              'among','an','and','any','are','as','at','be','because',
              'been','but','by','can','cannot','could','dear','did','do',
              'does','either','else','ever','every','for','from','get',
              'got','had','has','have','he','her','hers','him','his','how',
              'however','i','if','in','into','is','it','its','just','least',
              'let','like','likely','may','me','might','most','must','my',
              'neither','no','nor','not','of','off','often','on','only','or',
              'other','our','own','rather','said','say','says','she','should',
              'since','so','some','than','that','the','their','them','then',
              'there','these','they','this','tis','to','too','twas','us','wants',
              'was','we','were','what','when','where','which','while','who',
              'whom','why','will','with','would','yet','you','your',
              'find','very','still','non','here', 'many', 'a','s','t','ve', 
              'use', 'don\'t', 'can\'t', 'wont', 'come','you\'ll', 'want']

template_directory = "templates"
THEME_ROOT = get_theme_root()

# this function is not necessary - datetime.now() *is* localized in django
def now_localized():
    from datetime import datetime
    from timezones.utils import adjust_datetime_to_timezone
    from time import strftime, gmtime
    
    os_timezone = strftime('%Z',gmtime())
    if os_timezone == 'CST': os_timezone = 'US/Central'
    django_timezone =  settings.TIME_ZONE

    now = adjust_datetime_to_timezone(
               datetime.now(),
               from_tz=os_timezone,
               to_tz=django_timezone)
    now = now.replace(tzinfo=None)
    return now


def localize_date(date, from_tz=None, to_tz=None):
    """
        Takes the given date/timezone
        and converts it to the sites date/timezone

        localize_date(date, from_tz, to_tz=None)
    """
    from timezones.utils import adjust_datetime_to_timezone
    
    # set the defaults
    if from_tz is None:
        from_tz=settings.TIME_ZONE
    
    if to_tz is None: 
        to_tz=settings.TIME_ZONE
        
    return adjust_datetime_to_timezone(date,from_tz=from_tz,to_tz=to_tz)

def tcurrency(mymoney):
    """
        format currency - GJQ
        ex: 30000.232 -> $30,000.23
            -30000.232 -> $(30,000.23)
    """
    import locale
    locale.setlocale(locale.LC_ALL, '')
    currency_symbol = get_setting("site", "global", "currencysymbol")
   
    if not currency_symbol: currency_symbol = "$"

    if not isinstance(mymoney, str):
        if mymoney >= 0:
            return currency_symbol + locale.format('%.2f', mymoney, True)
        else:
            return currency_symbol + '(%s)' % (locale.format('%.2f', abs(mymoney), True))
    else:
        return mymoney
    
def format_datetime_range(start_dt, end_dt, format_date='%A, %B %d, %Y', format_time='%I:%M %p'):
    """
        takes datetime objects, start_dt, end_dt and format (for date and time) 
        returns a formated datetime string with range.
        ex:
            dt_str = format_datetime_range(datetime(2010, 8, 12, 8, 30, 0), 
                                           datetime(2010, 8, 12, 17, 30, 0))
            # returns: Thursday, August 12, 2010 8:30 AM - 05:30 PM
                                                                    - GJQ 8/12/2010
    """
    if isinstance(start_dt, datetime) and isinstance(end_dt, datetime):
        if start_dt.date() == end_dt.date():
            return '%s %s - %s' % (start_dt.strftime(format_date), 
                                   start_dt.strftime(format_time), 
                                   end_dt.strftime(format_time))
        else:
            return '%s %s - %s %s' % (start_dt.strftime(format_date), 
                                      start_dt.strftime(format_time),
                                      end_dt.strftime(format_date),
                                      end_dt.strftime(format_time))
            
            
def day_validate(dt, day):
    """
        validate if this day is valid in the month of dt, and correct it if not.
    """
    if isinstance(dt, datetime):
        try:
            day = int(day)
        except:
            day = 1
            
        if day == 0: day = 1
        
        # the last day of this month
        last_day_of_month = (datetime(dt.year, dt.month, 1) + relativedelta(months=1) - timedelta(days=1)).day
         
        # if last_day_of_month = 31 and day = 32, set day = 31   
        if day > last_day_of_month:
            day = last_day_of_month
    return day
        
            
def get_unique_username(user):
    import uuid
    from django.contrib.auth.models import User
    if not user.username:
        if user.email:
            user.username = user.email
    if not user.username:
        if user.first_name and user.last_name:
            user.username = '%s%s' % (user.first_name[0], user.last_name)
    if not user.username:
        user.username = str(uuid.uuid1())[:7]
    if len(user.username) > 20:
        user.username = user.username[:7]
        
    # check if this username already exists
    users = User.objects.filter(username__istartswith=user.username)
    
    if users:
        t_list = [u.username[len(user.username):] for u in users]
        num = 1
        while str(num) in t_list:
            num += 1
            
        user.username = '%s%s' % (user.username, str(num))
   
    return user.username


def send_email_notification(notice_name, recipients, extra_context):
    """
    Send email notice specified by the notice_name to the recipients.
    recipients - a list of emails
    """
    try:
        from tendenci.apps.notifications import models as notification
    except:
        notification = None
    if notification and recipients:
        notification.send_emails(recipients, notice_name, extra_context)
    
                                       
def generate_meta_keywords(value):
    """ 
        Take any string and removes the html and html entities
        and then runs a keyword density analyizer on the text
        to decided the 20 best one word and two word
        key phrases
    """
    try:
        from re import compile
        from operator import itemgetter
       
        from django.utils.html import strip_tags
        from django.utils.text import unescape_entities
        from django.utils.translation import ugettext_lazy as _
        
        # translate the stop words
        TR_STOP_WORDS = _(' '.join(STOP_WORDS))
        TR_STOP_WORDS = TR_STOP_WORDS.split()
        
        # get rid of the html tags
        value = strip_tags(value)
        
        # get rid of the html entities
        value = unescape_entities(value)
        
        # lower case the value
        value = value.lower()
    
        # get the one word, two word, and three word patterns
        one_word_pattern = compile(r'\s*(\w+[a-zA-Z0-9:\-]*\w*(\'\w{1,2})?)')
        two_word_pattern = compile(r'\s*(\w+[a-zA-Z0-9:\-]*\w*(\'\w{1,2})?)(\s+|_)(\w+[a-zA-Z0-9:\-]*\w*(\'\w{1,2})?)')
    
        # get the length of the value
        value_length = len(value)
        
        # make a list of one words
        search_end = 0
        one_words = []
        while search_end < value_length:
            s = one_word_pattern.search(value, search_end)
            if s:
                word = s.group(1)
                # remove the : from the word
                if word[-1] == ':':
                    word = word[:-1]
                    
                one_words.append(word)
                search_end = s.end()            
            else: break
    
        # remove the stop words
        one_words = [word for word in one_words if word not in TR_STOP_WORDS]

        # get the density, and word into a tuple
        one_words_length = len(one_words)
        unique_words = set(word for word in one_words)
        one_words = [(word, round((one_words.count(word)*100.00/one_words_length),2)) for word in unique_words]

        # sort the tuple by the density
        one_words = sorted(one_words, key=itemgetter(1), reverse=True)
        
        # get the 10 best keywords
        one_words = [word[0] for word in one_words[:10]]
        
        # make a list of two words phrases without stop phrases
        search_end = 0  
        two_words = []
        while search_end < value_length:
            s = two_word_pattern.search(value, search_end)
            if s:
                word1 = s.group(1)
                word2 = s.group(4)
                # remove the : from the words
                if word1[-1] == ':':
                    word1 = word1[:-1]
                if word2[-1] == ':':
                    word2 = word2[:-1]
                
                if word1 not in TR_STOP_WORDS:
                    if word2 not in TR_STOP_WORDS:
                        two_word = word1 + ' ' + word2
                        two_words.append(two_word)
                                
                search_start = s.start()
                next_search = one_word_pattern.search(value, search_start)
                search_end = next_search.end()   
            else:
                # if no match, advance a word
                s = one_word_pattern.search(value, search_end)
                if s:
                    search_end = s.end()
                else: search_end = value_length
          
        # get the density, and word into a tuple
        two_words_length = len(two_words)
        unique_words = set(words for words in two_words)
        two_words = [(words, round((two_words.count(words)*100.00/two_words_length),2)) for words in unique_words]

        # sort the tuple by the density
        two_words = sorted(two_words, key=itemgetter(1), reverse=True)
        
        # get the best 2 word keywords
        two_words = [word[0] for word in two_words[:10]]
        
        # add the two lists together
        keywords = two_words + one_words
        
        return ','.join(keywords)           
    except AttributeError:
        return ''  

def filelog(*args, **kwargs):
    """
        Will generate a file with output to the
        PROJECT_ROOT
        
        filelog(args...Nargs)
        filelog(args, mode='a+', filename='log.txt', path=path)
    """
    if kwargs.has_key('path'):
        path = kwargs['path']
    else:
        path = getattr(settings,'PROJECT_ROOT','/var/log')
        
    if kwargs.has_key('filename'):
        filename = kwargs['filename']
    else:
        filename = 'filelog.txt'
        
    if kwargs.has_key('mode'):
        mode = kwargs['mode']
    else:
        mode = 'a+'
        
    f = open(path + '/%s' % filename, mode)
    for arg in args:
        f.write(arg)
    f.close()    
        
class FormDateTimes(object):
    """
        Object that creates the start and end dates and times
        for a form that needs them
    """
    def __init__(self, start_dt=None, end_dt=None):
        self.start_dt = start_dt
        self.end_dt = end_dt
        self.get_date_times()
    def get_date_times(self):    
        if self.start_dt is None:
            self.start_dt = datetime.now()
        
        # remove the seconds and microseconds
        self.start_dt = self.start_dt.replace(second=00,microsecond=00)
             
        # check if its past the half hour or before and increment the time. 
        if self.start_dt.minute > 0 and self.start_dt.minute < 30:
            self.start_dt = self.start_dt.replace(minute=30)
        if self.start_dt.minute > 30:
            if self.start_dt.hour >= 23:
                self.start_dt = self.start_dt.replace(hour=00)
            else:
                self.start_dt = self.start_dt.replace(hour=self.start_dt.hour+1)
            
            self.start_dt = self.start_dt.replace(minute=00)    
        
        # set the end time to an hour ahead
        self.end_dt = self.start_dt + timedelta(hours=1)

date_times = FormDateTimes()
    
def enc_pass(password):
    from base64 import urlsafe_b64encode
    return ''.join(list(reversed(urlsafe_b64encode(password))))

def dec_pass(password):
    from base64 import urlsafe_b64decode

    pw_list = list(str(password))
    pw_list.reverse()
    
    return urlsafe_b64decode(''.join(pw_list))

def url_exists(url):
    import socket
    import httplib
    from parse_uri import ParseUri
    from django.contrib.sites.models import Site
    
    # parse url
    p = ParseUri()
    parsed_url = p.parse(url)
    
    # doesn't have a host so it's relative to the website
    if not parsed_url.host: 
        parsed_url = p.parse(Site.objects.get_current().domain + url)
    
    conn = httplib.HTTPConnection(parsed_url.authority)
    conn.request("HEAD", parsed_url.path)
    
    try:
        socket.setdefaulttimeout(1.5)
        response = conn.getresponse()
        if response.status == 200:
            return True
    except:
        return False

def parse_image_sources(string):
    import re
    p = re.compile('<img[^>]* src=\"([^\"]*)\"[^>]*>')
    image_sources = re.findall(p, string)
    return image_sources

def make_image_object_from_url(image_url):
    import urllib2
    import socket
    from parse_uri import ParseUri
    from PIL import Image
    from StringIO import StringIO
    from django.contrib.sites.models import Site

    # parse url
    p = ParseUri()
    parsed_url = p.parse(image_url)
    
    # handle absolute and relative urls, Assuming http for now. 
    if not parsed_url.host: 
        parsed_url = p.parse(Site.objects.get_current().domain + image_url)
        parsed_url.protocol = "http"
        parsed_url.source = parsed_url.protocol + "://" + parsed_url.source

    request = urllib2.Request(parsed_url.source)
    request.add_header('User-Agent', settings.TENDENCI_USER_AGENT)
    opener = urllib2.build_opener()

    # make image object
    try:
        socket.setdefaulttimeout(1.5)
        data = opener.open(request).read() # get data 
        im = Image.open(StringIO(data))
    except: 
        im = None
    return im

def image_rescale(img, size, force=True):
    """Rescale the given image, optionally cropping it to make sure the result image has the specified width and height."""
    import Image as pil
    
    max_width, max_height = size

    if not force:
        img.thumbnail((max_width, max_height), pil.ANTIALIAS)
    else:
        src_width, src_height = img.size
        src_ratio = float(src_width) / float(src_height)
        dst_width, dst_height = max_width, max_height
        dst_ratio = float(dst_width) / float(dst_height)
        
        if dst_ratio < src_ratio:
            crop_height = src_height
            crop_width = crop_height * dst_ratio
            x_offset = float(src_width - crop_width) / 2
            y_offset = 0
        else:
            crop_width = src_width
            crop_height = crop_width / dst_ratio
            x_offset = 0
            y_offset = float(src_height - crop_height) / 3
        img = img.crop((int(x_offset), int(y_offset), int(x_offset)+int(crop_width), int(y_offset)+int(crop_height)))
        img = img.resize((dst_width, dst_height), pil.ANTIALIAS)
    
    return img

def in_group(user, group):
    """
        Tells you if a user is in a particular group
        in_group(user,'administrator')
        returns boolean
    """
    return group in [dict['name'].lower() for dict in user.groups.values('name')]

def detect_template_tags(string):
    """
        Used to detect wether or not any string contains
        template tags in the system
        returns boolean
    """
    import re
    p = re.compile('{[#{%][^#}%]+[%}#]}', re.IGNORECASE)
    return p.search(string)


def get_template_list():
    """
    Get a list of files from the template
    directory that begin with 'default-'
    """
    file_list = []
    theme = get_setting('module', 'theme_editor', 'theme')
    if hasattr(settings, 'USE_S3_STORAGE') and settings.USE_S3_STORAGE:
        theme_dir = settings.ORIGINAL_THEMES_DIR
    else:
        theme_dir = settings.THEMES_DIR

    current_dir = os.path.join(theme_dir, theme, template_directory)

    if os.path.isdir(current_dir):
        item_list = os.listdir(current_dir)
    else:
        item_list = []

    for item in item_list:
        current_item = os.path.join(current_dir, item)
        path_split = os.path.splitext(current_item)
        extension = path_split[1]
        base_name = os.path.basename(path_split[0])
        if os.path.isfile(current_item):
            if extension == ".html" and "default-" in base_name:
                base_display_name = base_name[8:].replace('-',' ').title()
                file_list.append((item,base_display_name,))
    return sorted(file_list)


def check_template(filename):
    """
    Check to see if the file exists in the theme root
    """
    current_file = os.path.join(settings.ORIGINAL_THEMES_DIR, THEME_ROOT, template_directory, filename)
    return os.path.isfile(current_file)

def template_exists(template):
    """
    Check if the template exists
    """
    try:
        get_template(template)
    except TemplateDoesNotExist:
        return False
    return True

def fieldify(str):
    """Convert the fields in the square brackets to the django field type. 
    
        Example: "[First Name]: Lisa" 
                    will be converted to 
                "{{ first_name }}: Lisa"
    """
    #p = re.compile('(\[([\w\d\s_-]+)\])')
    p = re.compile('(\[(.*?)\])')
    return p.sub(slugify_fields, str)

def slugify_fields(match):
    return '{{ %s }}' % (slugify(match.group(2))).replace('-', '_')


def convert_absolute_urls(content, base_url):
    """
    Convert the relative urls in the content to the absolute urls.
    """
    content = content.replace('src="/', 'src="%s/' % base_url)
    content = content.replace('href="/', 'href="%s/' % base_url)
    return content


def is_blank(item):
    """
    Check if values inside list are blank
    Check if values inside dictionary are blank.
    """
    if isinstance(item, dict):
        l = item.values()
    elif isinstance(item, list):
        l = item
    elif isinstance(item, basestring):
        l = list(item)

    # return not bool([i for i in l if i.strip)
    return not bool(''.join(l).strip())

    raise Exception


def normalize_newline(file_path):
    """
    Normalize the new lines for a file ```file_path```

    ```file_path``` is a relative path.
    """
    data = default_storage.open(file_path).read()
    data = data.replace('\r\n', '\n').replace('\r', '\n')
    f = default_storage.open(file_path, 'w')
    f.write(data)
    f.close()




from django.core.management.base import BaseCommand, CommandError
from django.db.models.loading import get_model


class Command(BaseCommand):
    args = '<export_pk, field, field, field...>'
    help = "Runs an export task for the specified model."

    def handle(self, *args, **options):
        from tendenci.core.exports.models import Export
        from tendenci.core.exports.tasks import TendenciExportTask
        if args:

            try:
                export = Export.objects.get(pk=int(args[0]))
            except Export.DoesNotExist:
                raise CommandError('Export not specified')

            self.stdout.write('Started compiling export file...')

            # special case models
            if export.app_label == 'events' and export.model_name == 'event':
                from tendenci.addons.events.tasks import EventsExportTask
                result = EventsExportTask()
                response = result.run()
            elif export.app_label == 'forms_builder.forms' and export.model_name == 'form':
                from tendenci.apps.forms_builder.forms.tasks import FormsExportTask
                result = FormsExportTask()
                response = result.run()
            elif export.app_label == 'navs' and export.model_name == 'nav':
                from tendenci.apps.navs.tasks import NavsExportTask
                result = NavsExportTask()
                response = result.run()
            elif export.app_label == 'pages' and export.model_name == 'page':
                from tendenci.apps.pages.tasks import PagesExportTask
                result = PagesExportTask()
                response = result.run()
            elif export.app_label == 'memberships' and export.model_name == 'membership':
                from tendenci.addons.memberships.tasks import MembershipsExportTask
                kwargs = {'app': export.memb_app}
                result = MembershipsExportTask()
                response = result.run(**kwargs)
            else:
                model = get_model(export.app_label, export.model_name)
                result = TendenciExportTask()
                file_name = export.model_name + '.csv'
                response = result.run(model, args[1:], file_name)

            export.status = "completed"
            export.result = response
            export.save()

            self.stdout.write('Successfully completed export file.')
        else:
            raise CommandError('Export args not specified')
